# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "storage account network-security-perimeter-configuration show",
)
class Show(AAZCommand):
    """Get effective NetworkSecurityPerimeterConfiguration for association

    :example: Get effective NetworkSecurityPerimeterConfiguration for association
        az storage account network-security-perimeter-configuration show --resource-group res4410 --account-name sto8607 --network-security-perimeter-configuration-name dbedb4e0-40e6-4145-81f3-f1314c150774.resourceAssociation1
    """

    _aaz_info = {
        "version": "2025-01-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.storage/storageaccounts/{}/networksecurityperimeterconfigurations/{}", "2025-01-01"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.account_name = AAZStrArg(
            options=["--account-name"],
            help="The name of the storage account within the specified resource group. Storage account names must be between 3 and 24 characters in length and use numbers and lower-case letters only.",
            required=True,
            id_part="name",
            fmt=AAZStrArgFormat(
                pattern="^[a-z0-9]+$",
                max_length=24,
                min_length=3,
            ),
        )
        _args_schema.network_security_perimeter_configuration_name = AAZStrArg(
            options=["-n", "--name", "--network-security-perimeter-configuration-name"],
            help="The name for Network Security Perimeter configuration",
            required=True,
            id_part="child_name_1",
            fmt=AAZStrArgFormat(
                pattern="^.*$",
            ),
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.NetworkSecurityPerimeterConfigurationsGet(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class NetworkSecurityPerimeterConfigurationsGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Storage/storageAccounts/{accountName}/networkSecurityPerimeterConfigurations/{networkSecurityPerimeterConfigurationName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "MgmtErrorFormat"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "accountName", self.ctx.args.account_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "networkSecurityPerimeterConfigurationName", self.ctx.args.network_security_perimeter_configuration_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2025-01-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType(
                flags={"read_only": True},
            )

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.id = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.properties = AAZObjectType(
                flags={"client_flatten": True, "read_only": True},
            )
            _schema_on_200.system_data = AAZObjectType(
                serialized_name="systemData",
                flags={"read_only": True},
            )
            _schema_on_200.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.properties
            properties.network_security_perimeter = AAZObjectType(
                serialized_name="networkSecurityPerimeter",
                flags={"read_only": True},
            )
            _ShowHelper._build_schema_network_security_perimeter_read(properties.network_security_perimeter)
            properties.profile = AAZObjectType(
                flags={"read_only": True},
            )
            properties.provisioning_issues = AAZListType(
                serialized_name="provisioningIssues",
                flags={"read_only": True},
            )
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.resource_association = AAZObjectType(
                serialized_name="resourceAssociation",
                flags={"read_only": True},
            )

            profile = cls._schema_on_200.properties.profile
            profile.access_rules = AAZListType(
                serialized_name="accessRules",
            )
            profile.access_rules_version = AAZFloatType(
                serialized_name="accessRulesVersion",
            )
            profile.diagnostic_settings_version = AAZFloatType(
                serialized_name="diagnosticSettingsVersion",
            )
            profile.enabled_log_categories = AAZListType(
                serialized_name="enabledLogCategories",
            )
            profile.name = AAZStrType()

            access_rules = cls._schema_on_200.properties.profile.access_rules
            access_rules.Element = AAZObjectType(
                flags={"read_only": True},
            )

            _element = cls._schema_on_200.properties.profile.access_rules.Element
            _element.name = AAZStrType()
            _element.properties = AAZObjectType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.properties.profile.access_rules.Element.properties
            properties.address_prefixes = AAZListType(
                serialized_name="addressPrefixes",
            )
            properties.direction = AAZStrType()
            properties.fully_qualified_domain_names = AAZListType(
                serialized_name="fullyQualifiedDomainNames",
                flags={"read_only": True},
            )
            properties.network_security_perimeters = AAZListType(
                serialized_name="networkSecurityPerimeters",
                flags={"read_only": True},
            )
            properties.subscriptions = AAZListType()

            address_prefixes = cls._schema_on_200.properties.profile.access_rules.Element.properties.address_prefixes
            address_prefixes.Element = AAZStrType()

            fully_qualified_domain_names = cls._schema_on_200.properties.profile.access_rules.Element.properties.fully_qualified_domain_names
            fully_qualified_domain_names.Element = AAZStrType()

            network_security_perimeters = cls._schema_on_200.properties.profile.access_rules.Element.properties.network_security_perimeters
            network_security_perimeters.Element = AAZObjectType(
                flags={"read_only": True},
            )
            _ShowHelper._build_schema_network_security_perimeter_read(network_security_perimeters.Element)

            subscriptions = cls._schema_on_200.properties.profile.access_rules.Element.properties.subscriptions
            subscriptions.Element = AAZObjectType(
                flags={"read_only": True},
            )

            _element = cls._schema_on_200.properties.profile.access_rules.Element.properties.subscriptions.Element
            _element.id = AAZStrType()

            enabled_log_categories = cls._schema_on_200.properties.profile.enabled_log_categories
            enabled_log_categories.Element = AAZStrType()

            provisioning_issues = cls._schema_on_200.properties.provisioning_issues
            provisioning_issues.Element = AAZObjectType(
                flags={"read_only": True},
            )

            _element = cls._schema_on_200.properties.provisioning_issues.Element
            _element.name = AAZStrType()
            _element.properties = AAZObjectType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.properties.provisioning_issues.Element.properties
            properties.description = AAZStrType()
            properties.issue_type = AAZStrType(
                serialized_name="issueType",
            )
            properties.severity = AAZStrType()

            resource_association = cls._schema_on_200.properties.resource_association
            resource_association.access_mode = AAZStrType(
                serialized_name="accessMode",
            )
            resource_association.name = AAZStrType()

            system_data = cls._schema_on_200.system_data
            system_data.created_at = AAZStrType(
                serialized_name="createdAt",
            )
            system_data.created_by = AAZStrType(
                serialized_name="createdBy",
            )
            system_data.created_by_type = AAZStrType(
                serialized_name="createdByType",
            )
            system_data.last_modified_at = AAZStrType(
                serialized_name="lastModifiedAt",
            )
            system_data.last_modified_by = AAZStrType(
                serialized_name="lastModifiedBy",
            )
            system_data.last_modified_by_type = AAZStrType(
                serialized_name="lastModifiedByType",
            )

            return cls._schema_on_200


class _ShowHelper:
    """Helper class for Show"""

    _schema_network_security_perimeter_read = None

    @classmethod
    def _build_schema_network_security_perimeter_read(cls, _schema):
        if cls._schema_network_security_perimeter_read is not None:
            _schema.id = cls._schema_network_security_perimeter_read.id
            _schema.location = cls._schema_network_security_perimeter_read.location
            _schema.perimeter_guid = cls._schema_network_security_perimeter_read.perimeter_guid
            return

        cls._schema_network_security_perimeter_read = _schema_network_security_perimeter_read = AAZObjectType(
            flags={"read_only": True}
        )

        network_security_perimeter_read = _schema_network_security_perimeter_read
        network_security_perimeter_read.id = AAZStrType()
        network_security_perimeter_read.location = AAZStrType()
        network_security_perimeter_read.perimeter_guid = AAZStrType(
            serialized_name="perimeterGuid",
        )

        _schema.id = cls._schema_network_security_perimeter_read.id
        _schema.location = cls._schema_network_security_perimeter_read.location
        _schema.perimeter_guid = cls._schema_network_security_perimeter_read.perimeter_guid


__all__ = ["Show"]



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models_py3 import (  # type: ignore
    ApiError,
    ApiErrorBase,
    CommunityGallery,
    CommunityGalleryImage,
    CommunityGalleryImageList,
    CommunityGalleryImageVersion,
    CommunityGalleryImageVersionList,
    CommunityGalleryInfo,
    DataDiskImageEncryption,
    Disallowed,
    DiskImageEncryption,
    EncryptionImages,
    ExtendedLocation,
    Gallery,
    GalleryApplication,
    GalleryApplicationList,
    GalleryApplicationUpdate,
    GalleryApplicationVersion,
    GalleryApplicationVersionList,
    GalleryApplicationVersionPublishingProfile,
    GalleryApplicationVersionUpdate,
    GalleryArtifactPublishingProfileBase,
    GalleryArtifactSource,
    GalleryArtifactVersionSource,
    GalleryDataDiskImage,
    GalleryDiskImage,
    GalleryExtendedLocation,
    GalleryIdentifier,
    GalleryImage,
    GalleryImageFeature,
    GalleryImageIdentifier,
    GalleryImageList,
    GalleryImageUpdate,
    GalleryImageVersion,
    GalleryImageVersionList,
    GalleryImageVersionPublishingProfile,
    GalleryImageVersionStorageProfile,
    GalleryImageVersionUpdate,
    GalleryList,
    GalleryOSDiskImage,
    GalleryTargetExtendedLocation,
    GalleryUpdate,
    ImagePurchasePlan,
    InnerError,
    ManagedArtifact,
    OSDiskImageEncryption,
    OSDiskImageSecurityProfile,
    PirCommunityGalleryResource,
    PirResource,
    PirSharedGalleryResource,
    RecommendedMachineConfiguration,
    RegionalReplicationStatus,
    RegionalSharingStatus,
    ReplicationStatus,
    Resource,
    ResourceRange,
    ResourceWithOptionalLocation,
    SharedGallery,
    SharedGalleryDataDiskImage,
    SharedGalleryDiskImage,
    SharedGalleryImage,
    SharedGalleryImageList,
    SharedGalleryImageVersion,
    SharedGalleryImageVersionList,
    SharedGalleryImageVersionStorageProfile,
    SharedGalleryList,
    SharedGalleryOSDiskImage,
    SharingProfile,
    SharingProfileGroup,
    SharingStatus,
    SharingUpdate,
    SoftDeletePolicy,
    SubResource,
    SubResourceReadOnly,
    TargetRegion,
    UpdateResourceDefinition,
    UserArtifactManage,
    UserArtifactSettings,
    UserArtifactSource,
    UserAssignedIdentitiesValue,
)

from ._compute_management_client_enums import (  # type: ignore
    AggregatedReplicationState,
    Architecture,
    ConfidentialVMEncryptionType,
    ExtendedLocationTypes,
    GalleryExpandParams,
    GalleryExtendedLocationType,
    GalleryProvisioningState,
    GallerySharingPermissionTypes,
    HostCaching,
    HyperVGeneration,
    OperatingSystemStateTypes,
    OperatingSystemTypes,
    ReplicationMode,
    ReplicationState,
    ReplicationStatusTypes,
    SelectPermissions,
    SharedGalleryHostCaching,
    SharedToValues,
    SharingProfileGroupTypes,
    SharingState,
    SharingUpdateOperationTypes,
    StorageAccountType,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "ApiError",
    "ApiErrorBase",
    "CommunityGallery",
    "CommunityGalleryImage",
    "CommunityGalleryImageList",
    "CommunityGalleryImageVersion",
    "CommunityGalleryImageVersionList",
    "CommunityGalleryInfo",
    "DataDiskImageEncryption",
    "Disallowed",
    "DiskImageEncryption",
    "EncryptionImages",
    "ExtendedLocation",
    "Gallery",
    "GalleryApplication",
    "GalleryApplicationList",
    "GalleryApplicationUpdate",
    "GalleryApplicationVersion",
    "GalleryApplicationVersionList",
    "GalleryApplicationVersionPublishingProfile",
    "GalleryApplicationVersionUpdate",
    "GalleryArtifactPublishingProfileBase",
    "GalleryArtifactSource",
    "GalleryArtifactVersionSource",
    "GalleryDataDiskImage",
    "GalleryDiskImage",
    "GalleryExtendedLocation",
    "GalleryIdentifier",
    "GalleryImage",
    "GalleryImageFeature",
    "GalleryImageIdentifier",
    "GalleryImageList",
    "GalleryImageUpdate",
    "GalleryImageVersion",
    "GalleryImageVersionList",
    "GalleryImageVersionPublishingProfile",
    "GalleryImageVersionStorageProfile",
    "GalleryImageVersionUpdate",
    "GalleryList",
    "GalleryOSDiskImage",
    "GalleryTargetExtendedLocation",
    "GalleryUpdate",
    "ImagePurchasePlan",
    "InnerError",
    "ManagedArtifact",
    "OSDiskImageEncryption",
    "OSDiskImageSecurityProfile",
    "PirCommunityGalleryResource",
    "PirResource",
    "PirSharedGalleryResource",
    "RecommendedMachineConfiguration",
    "RegionalReplicationStatus",
    "RegionalSharingStatus",
    "ReplicationStatus",
    "Resource",
    "ResourceRange",
    "ResourceWithOptionalLocation",
    "SharedGallery",
    "SharedGalleryDataDiskImage",
    "SharedGalleryDiskImage",
    "SharedGalleryImage",
    "SharedGalleryImageList",
    "SharedGalleryImageVersion",
    "SharedGalleryImageVersionList",
    "SharedGalleryImageVersionStorageProfile",
    "SharedGalleryList",
    "SharedGalleryOSDiskImage",
    "SharingProfile",
    "SharingProfileGroup",
    "SharingStatus",
    "SharingUpdate",
    "SoftDeletePolicy",
    "SubResource",
    "SubResourceReadOnly",
    "TargetRegion",
    "UpdateResourceDefinition",
    "UserArtifactManage",
    "UserArtifactSettings",
    "UserArtifactSource",
    "UserAssignedIdentitiesValue",
    "AggregatedReplicationState",
    "Architecture",
    "ConfidentialVMEncryptionType",
    "ExtendedLocationTypes",
    "GalleryExpandParams",
    "GalleryExtendedLocationType",
    "GalleryProvisioningState",
    "GallerySharingPermissionTypes",
    "HostCaching",
    "HyperVGeneration",
    "OperatingSystemStateTypes",
    "OperatingSystemTypes",
    "ReplicationMode",
    "ReplicationState",
    "ReplicationStatusTypes",
    "SelectPermissions",
    "SharedGalleryHostCaching",
    "SharedToValues",
    "SharingProfileGroupTypes",
    "SharingState",
    "SharingUpdateOperationTypes",
    "StorageAccountType",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

import datetime
from typing import Dict, List, Optional, Union

from azure.core.exceptions import HttpResponseError
import msrest.serialization

from ._azure_table_enums import *


class AccessPolicy(msrest.serialization.Model):
    """An Access policy.

    All required parameters must be populated in order to send to Azure.

    :param start: Required. The start datetime from which the policy is active.
    :type start: str
    :param expiry: Required. The datetime that the policy expires.
    :type expiry: str
    :param permission: Required. The permissions for the acl policy.
    :type permission: str
    """

    _validation = {
        'start': {'required': True},
        'expiry': {'required': True},
        'permission': {'required': True},
    }

    _attribute_map = {
        'start': {'key': 'Start', 'type': 'str', 'xml': {'name': 'Start'}},
        'expiry': {'key': 'Expiry', 'type': 'str', 'xml': {'name': 'Expiry'}},
        'permission': {'key': 'Permission', 'type': 'str', 'xml': {'name': 'Permission'}},
    }
    _xml_map = {
        'name': 'AccessPolicy'
    }

    def __init__(
        self,
        *,
        start: str,
        expiry: str,
        permission: str,
        **kwargs
    ):
        super(AccessPolicy, self).__init__(**kwargs)
        self.start = start
        self.expiry = expiry
        self.permission = permission


class CorsRule(msrest.serialization.Model):
    """CORS is an HTTP feature that enables a web application running under one domain to access resources in another domain. Web browsers implement a security restriction known as same-origin policy that prevents a web page from calling APIs in a different domain; CORS provides a secure way to allow one domain (the origin domain) to call APIs in another domain.

    All required parameters must be populated in order to send to Azure.

    :param allowed_origins: Required. The origin domains that are permitted to make a request
     against the service via CORS. The origin domain is the domain from which the request
     originates. Note that the origin must be an exact case-sensitive match with the origin that the
     user age sends to the service. You can also use the wildcard character '*' to allow all origin
     domains to make requests via CORS.
    :type allowed_origins: str
    :param allowed_methods: Required. The methods (HTTP request verbs) that the origin domain may
     use for a CORS request. (comma separated).
    :type allowed_methods: str
    :param allowed_headers: Required. The request headers that the origin domain may specify on the
     CORS request.
    :type allowed_headers: str
    :param exposed_headers: Required. The response headers that may be sent in the response to the
     CORS request and exposed by the browser to the request issuer.
    :type exposed_headers: str
    :param max_age_in_seconds: Required. The maximum amount time that a browser should cache the
     preflight OPTIONS request.
    :type max_age_in_seconds: int
    """

    _validation = {
        'allowed_origins': {'required': True},
        'allowed_methods': {'required': True},
        'allowed_headers': {'required': True},
        'exposed_headers': {'required': True},
        'max_age_in_seconds': {'required': True, 'minimum': 0},
    }

    _attribute_map = {
        'allowed_origins': {'key': 'AllowedOrigins', 'type': 'str', 'xml': {'name': 'AllowedOrigins'}},
        'allowed_methods': {'key': 'AllowedMethods', 'type': 'str', 'xml': {'name': 'AllowedMethods'}},
        'allowed_headers': {'key': 'AllowedHeaders', 'type': 'str', 'xml': {'name': 'AllowedHeaders'}},
        'exposed_headers': {'key': 'ExposedHeaders', 'type': 'str', 'xml': {'name': 'ExposedHeaders'}},
        'max_age_in_seconds': {'key': 'MaxAgeInSeconds', 'type': 'int', 'xml': {'name': 'MaxAgeInSeconds'}},
    }
    _xml_map = {
        'name': 'CorsRule'
    }

    def __init__(
        self,
        *,
        allowed_origins: str,
        allowed_methods: str,
        allowed_headers: str,
        exposed_headers: str,
        max_age_in_seconds: int,
        **kwargs
    ):
        super(CorsRule, self).__init__(**kwargs)
        self.allowed_origins = allowed_origins
        self.allowed_methods = allowed_methods
        self.allowed_headers = allowed_headers
        self.exposed_headers = exposed_headers
        self.max_age_in_seconds = max_age_in_seconds


class GeoReplication(msrest.serialization.Model):
    """GeoReplication.

    All required parameters must be populated in order to send to Azure.

    :param status: Required. The status of the secondary location. Possible values include: "live",
     "bootstrap", "unavailable".
    :type status: str or ~azure.data.tables.models.GeoReplicationStatusType
    :param last_sync_time: Required. A GMT date/time value, to the second. All primary writes
     preceding this value are guaranteed to be available for read operations at the secondary.
     Primary writes after this point in time may or may not be available for reads.
    :type last_sync_time: ~datetime.datetime
    """

    _validation = {
        'status': {'required': True},
        'last_sync_time': {'required': True},
    }

    _attribute_map = {
        'status': {'key': 'Status', 'type': 'str', 'xml': {'name': 'Status'}},
        'last_sync_time': {'key': 'LastSyncTime', 'type': 'rfc-1123', 'xml': {'name': 'LastSyncTime'}},
    }
    _xml_map = {
        'name': 'GeoReplication'
    }

    def __init__(
        self,
        *,
        status: Union[str, "GeoReplicationStatusType"],
        last_sync_time: datetime.datetime,
        **kwargs
    ):
        super(GeoReplication, self).__init__(**kwargs)
        self.status = status
        self.last_sync_time = last_sync_time


class Logging(msrest.serialization.Model):
    """Azure Analytics Logging settings.

    All required parameters must be populated in order to send to Azure.

    :param version: Required. The version of Analytics to configure.
    :type version: str
    :param delete: Required. Indicates whether all delete requests should be logged.
    :type delete: bool
    :param read: Required. Indicates whether all read requests should be logged.
    :type read: bool
    :param write: Required. Indicates whether all write requests should be logged.
    :type write: bool
    :param retention_policy: Required. The retention policy.
    :type retention_policy: ~azure.data.tables.models.RetentionPolicy
    """

    _validation = {
        'version': {'required': True},
        'delete': {'required': True},
        'read': {'required': True},
        'write': {'required': True},
        'retention_policy': {'required': True},
    }

    _attribute_map = {
        'version': {'key': 'Version', 'type': 'str', 'xml': {'name': 'Version'}},
        'delete': {'key': 'Delete', 'type': 'bool', 'xml': {'name': 'Delete'}},
        'read': {'key': 'Read', 'type': 'bool', 'xml': {'name': 'Read'}},
        'write': {'key': 'Write', 'type': 'bool', 'xml': {'name': 'Write'}},
        'retention_policy': {'key': 'RetentionPolicy', 'type': 'RetentionPolicy'},
    }
    _xml_map = {
        'name': 'Logging'
    }

    def __init__(
        self,
        *,
        version: str,
        delete: bool,
        read: bool,
        write: bool,
        retention_policy: "RetentionPolicy",
        **kwargs
    ):
        super(Logging, self).__init__(**kwargs)
        self.version = version
        self.delete = delete
        self.read = read
        self.write = write
        self.retention_policy = retention_policy


class Metrics(msrest.serialization.Model):
    """Metrics.

    All required parameters must be populated in order to send to Azure.

    :param version: The version of Analytics to configure.
    :type version: str
    :param enabled: Required. Indicates whether metrics are enabled for the Table service.
    :type enabled: bool
    :param include_apis: Indicates whether metrics should generate summary statistics for called
     API operations.
    :type include_apis: bool
    :param retention_policy: The retention policy.
    :type retention_policy: ~azure.data.tables.models.RetentionPolicy
    """

    _validation = {
        'enabled': {'required': True},
    }

    _attribute_map = {
        'version': {'key': 'Version', 'type': 'str', 'xml': {'name': 'Version'}},
        'enabled': {'key': 'Enabled', 'type': 'bool', 'xml': {'name': 'Enabled'}},
        'include_apis': {'key': 'IncludeAPIs', 'type': 'bool', 'xml': {'name': 'IncludeAPIs'}},
        'retention_policy': {'key': 'RetentionPolicy', 'type': 'RetentionPolicy'},
    }
    _xml_map = {
        
    }

    def __init__(
        self,
        *,
        enabled: bool,
        version: Optional[str] = None,
        include_apis: Optional[bool] = None,
        retention_policy: Optional["RetentionPolicy"] = None,
        **kwargs
    ):
        super(Metrics, self).__init__(**kwargs)
        self.version = version
        self.enabled = enabled
        self.include_apis = include_apis
        self.retention_policy = retention_policy


class QueryOptions(msrest.serialization.Model):
    """Parameter group.

    :param format: Specifies the media type for the response. Possible values include:
     "application/json;odata=nometadata", "application/json;odata=minimalmetadata",
     "application/json;odata=fullmetadata".
    :type format: str or ~azure.data.tables.models.OdataMetadataFormat
    :param top: Maximum number of records to return.
    :type top: int
    :param select: Select expression using OData notation. Limits the columns on each record to
     just those requested, e.g. "$select=PolicyAssignmentId, ResourceId".
    :type select: str
    :param filter: OData filter expression.
    :type filter: str
    """

    _validation = {
        'top': {'minimum': 0},
    }

    _attribute_map = {
        'format': {'key': 'Format', 'type': 'str'},
        'top': {'key': 'Top', 'type': 'int'},
        'select': {'key': 'Select', 'type': 'str'},
        'filter': {'key': 'Filter', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        format: Optional[Union[str, "OdataMetadataFormat"]] = None,
        top: Optional[int] = None,
        select: Optional[str] = None,
        filter: Optional[str] = None,
        **kwargs
    ):
        super(QueryOptions, self).__init__(**kwargs)
        self.format = format
        self.top = top
        self.select = select
        self.filter = filter


class RetentionPolicy(msrest.serialization.Model):
    """The retention policy.

    All required parameters must be populated in order to send to Azure.

    :param enabled: Required. Indicates whether a retention policy is enabled for the service.
    :type enabled: bool
    :param days: Indicates the number of days that metrics or logging or soft-deleted data should
     be retained. All data older than this value will be deleted.
    :type days: int
    """

    _validation = {
        'enabled': {'required': True},
        'days': {'minimum': 1},
    }

    _attribute_map = {
        'enabled': {'key': 'Enabled', 'type': 'bool', 'xml': {'name': 'Enabled'}},
        'days': {'key': 'Days', 'type': 'int', 'xml': {'name': 'Days'}},
    }
    _xml_map = {
        'name': 'RetentionPolicy'
    }

    def __init__(
        self,
        *,
        enabled: bool,
        days: Optional[int] = None,
        **kwargs
    ):
        super(RetentionPolicy, self).__init__(**kwargs)
        self.enabled = enabled
        self.days = days


class SignedIdentifier(msrest.serialization.Model):
    """A signed identifier.

    All required parameters must be populated in order to send to Azure.

    :param id: Required. A unique id.
    :type id: str
    :param access_policy: The access policy.
    :type access_policy: ~azure.data.tables.models.AccessPolicy
    """

    _validation = {
        'id': {'required': True},
    }

    _attribute_map = {
        'id': {'key': 'Id', 'type': 'str', 'xml': {'name': 'Id'}},
        'access_policy': {'key': 'AccessPolicy', 'type': 'AccessPolicy'},
    }
    _xml_map = {
        'name': 'SignedIdentifier'
    }

    def __init__(
        self,
        *,
        id: str,
        access_policy: Optional["AccessPolicy"] = None,
        **kwargs
    ):
        super(SignedIdentifier, self).__init__(**kwargs)
        self.id = id
        self.access_policy = access_policy


class TableEntityQueryResponse(msrest.serialization.Model):
    """The properties for the table entity query response.

    :param odata_metadata: The metadata response of the table.
    :type odata_metadata: str
    :param value: List of table entities.
    :type value: list[dict[str, object]]
    """

    _attribute_map = {
        'odata_metadata': {'key': 'odata\\.metadata', 'type': 'str'},
        'value': {'key': 'value', 'type': '[{object}]'},
    }

    def __init__(
        self,
        *,
        odata_metadata: Optional[str] = None,
        value: Optional[List[Dict[str, object]]] = None,
        **kwargs
    ):
        super(TableEntityQueryResponse, self).__init__(**kwargs)
        self.odata_metadata = odata_metadata
        self.value = value


class TableProperties(msrest.serialization.Model):
    """The properties for creating a table.

    :param table_name: The name of the table to create.
    :type table_name: str
    """

    _attribute_map = {
        'table_name': {'key': 'TableName', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        table_name: Optional[str] = None,
        **kwargs
    ):
        super(TableProperties, self).__init__(**kwargs)
        self.table_name = table_name


class TableQueryResponse(msrest.serialization.Model):
    """The properties for the table query response.

    :param odata_metadata: The metadata response of the table.
    :type odata_metadata: str
    :param value: List of tables.
    :type value: list[~azure.data.tables.models.TableResponseProperties]
    """

    _attribute_map = {
        'odata_metadata': {'key': 'odata\\.metadata', 'type': 'str'},
        'value': {'key': 'value', 'type': '[TableResponseProperties]'},
    }

    def __init__(
        self,
        *,
        odata_metadata: Optional[str] = None,
        value: Optional[List["TableResponseProperties"]] = None,
        **kwargs
    ):
        super(TableQueryResponse, self).__init__(**kwargs)
        self.odata_metadata = odata_metadata
        self.value = value


class TableResponseProperties(msrest.serialization.Model):
    """The properties for the table response.

    :param table_name: The name of the table.
    :type table_name: str
    :param odata_type: The odata type of the table.
    :type odata_type: str
    :param odata_id: The id of the table.
    :type odata_id: str
    :param odata_edit_link: The edit link of the table.
    :type odata_edit_link: str
    """

    _attribute_map = {
        'table_name': {'key': 'TableName', 'type': 'str'},
        'odata_type': {'key': 'odata\\.type', 'type': 'str'},
        'odata_id': {'key': 'odata\\.id', 'type': 'str'},
        'odata_edit_link': {'key': 'odata\\.editLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        table_name: Optional[str] = None,
        odata_type: Optional[str] = None,
        odata_id: Optional[str] = None,
        odata_edit_link: Optional[str] = None,
        **kwargs
    ):
        super(TableResponseProperties, self).__init__(**kwargs)
        self.table_name = table_name
        self.odata_type = odata_type
        self.odata_id = odata_id
        self.odata_edit_link = odata_edit_link


class TableResponse(TableResponseProperties):
    """The response for a single table.

    :param table_name: The name of the table.
    :type table_name: str
    :param odata_type: The odata type of the table.
    :type odata_type: str
    :param odata_id: The id of the table.
    :type odata_id: str
    :param odata_edit_link: The edit link of the table.
    :type odata_edit_link: str
    :param odata_metadata: The metadata response of the table.
    :type odata_metadata: str
    """

    _attribute_map = {
        'table_name': {'key': 'TableName', 'type': 'str'},
        'odata_type': {'key': 'odata\\.type', 'type': 'str'},
        'odata_id': {'key': 'odata\\.id', 'type': 'str'},
        'odata_edit_link': {'key': 'odata\\.editLink', 'type': 'str'},
        'odata_metadata': {'key': 'odata\\.metadata', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        table_name: Optional[str] = None,
        odata_type: Optional[str] = None,
        odata_id: Optional[str] = None,
        odata_edit_link: Optional[str] = None,
        odata_metadata: Optional[str] = None,
        **kwargs
    ):
        super(TableResponse, self).__init__(table_name=table_name, odata_type=odata_type, odata_id=odata_id, odata_edit_link=odata_edit_link, **kwargs)
        self.odata_metadata = odata_metadata


class TableServiceError(msrest.serialization.Model):
    """Table Service error.

    :param message: The error message.
    :type message: str
    """

    _attribute_map = {
        'message': {'key': 'Message', 'type': 'str', 'xml': {'name': 'Message'}},
    }
    _xml_map = {
        
    }

    def __init__(
        self,
        *,
        message: Optional[str] = None,
        **kwargs
    ):
        super(TableServiceError, self).__init__(**kwargs)
        self.message = message


class TableServiceProperties(msrest.serialization.Model):
    """Table Service Properties.

    :param logging: Azure Analytics Logging settings.
    :type logging: ~azure.data.tables.models.Logging
    :param hour_metrics: A summary of request statistics grouped by API in hourly aggregates for
     tables.
    :type hour_metrics: ~azure.data.tables.models.Metrics
    :param minute_metrics: A summary of request statistics grouped by API in minute aggregates for
     tables.
    :type minute_metrics: ~azure.data.tables.models.Metrics
    :param cors: The set of CORS rules.
    :type cors: list[~azure.data.tables.models.CorsRule]
    """

    _attribute_map = {
        'logging': {'key': 'Logging', 'type': 'Logging'},
        'hour_metrics': {'key': 'HourMetrics', 'type': 'Metrics'},
        'minute_metrics': {'key': 'MinuteMetrics', 'type': 'Metrics'},
        'cors': {'key': 'Cors', 'type': '[CorsRule]', 'xml': {'name': 'Cors', 'wrapped': True, 'itemsName': 'CorsRule'}},
    }
    _xml_map = {
        'name': 'StorageServiceProperties'
    }

    def __init__(
        self,
        *,
        logging: Optional["Logging"] = None,
        hour_metrics: Optional["Metrics"] = None,
        minute_metrics: Optional["Metrics"] = None,
        cors: Optional[List["CorsRule"]] = None,
        **kwargs
    ):
        super(TableServiceProperties, self).__init__(**kwargs)
        self.logging = logging
        self.hour_metrics = hour_metrics
        self.minute_metrics = minute_metrics
        self.cors = cors


class TableServiceStats(msrest.serialization.Model):
    """Stats for the service.

    :param geo_replication: Geo-Replication information for the Secondary Storage Service.
    :type geo_replication: ~azure.data.tables.models.GeoReplication
    """

    _attribute_map = {
        'geo_replication': {'key': 'GeoReplication', 'type': 'GeoReplication'},
    }
    _xml_map = {
        'name': 'StorageServiceStats'
    }

    def __init__(
        self,
        *,
        geo_replication: Optional["GeoReplication"] = None,
        **kwargs
    ):
        super(TableServiceStats, self).__init__(**kwargs)
        self.geo_replication = geo_replication



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class AccountKeyType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of account key to regenerate."""

    PRIMARY = "Primary"
    """The primary account key."""
    SECONDARY = "Secondary"
    """The secondary account key."""


class AllocationState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Whether the pool is resizing."""

    STEADY = "Steady"
    """The pool is not resizing. There are no changes to the number of nodes in the pool in progress.
    #: A pool enters this state when it is created and when no operations are being performed on the
    #: pool to change the number of nodes."""
    RESIZING = "Resizing"
    """The pool is resizing; that is, compute nodes are being added to or removed from the pool."""
    STOPPING = "Stopping"
    """The pool was resizing, but the user has requested that the resize be stopped, but the stop
    #: request has not yet been completed."""


class AuthenticationMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The authentication mode for the Batch account."""

    SHARED_KEY = "SharedKey"
    """The authentication mode using shared keys."""
    AAD = "AAD"
    """The authentication mode using Microsoft Entra ID."""
    TASK_AUTHENTICATION_TOKEN = "TaskAuthenticationToken"
    """The authentication mode using task authentication tokens."""


class AutoStorageAuthenticationMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The authentication mode which the Batch service will use to manage the auto-storage account."""

    STORAGE_KEYS = "StorageKeys"
    """The Batch service will authenticate requests to auto-storage using storage account keys."""
    BATCH_ACCOUNT_MANAGED_IDENTITY = "BatchAccountManagedIdentity"
    """The Batch service will authenticate requests to auto-storage using the managed identity
    #: assigned to the Batch account."""


class AutoUserScope(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The default value is Pool. If the pool is running Windows a value of Task should be specified
    if stricter isolation between tasks is required. For example, if the task mutates the registry
    in a way which could impact other tasks, or if certificates have been specified on the pool
    which should not be accessible by normal tasks but should be accessible by start tasks.
    """

    TASK = "Task"
    """Specifies that the service should create a new user for the task."""
    POOL = "Pool"
    """Specifies that the task runs as the common auto user account which is created on every node in
    #: a pool."""


class CachingType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of caching to enable for the disk."""

    NONE = "None"
    """The caching mode for the disk is not enabled."""
    READ_ONLY = "ReadOnly"
    """The caching mode for the disk is read only."""
    READ_WRITE = "ReadWrite"
    """The caching mode for the disk is read and write."""


class CertificateFormat(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The format of the certificate - either Pfx or Cer. If omitted, the default is Pfx."""

    PFX = "Pfx"
    """The certificate is a PFX (PKCS#12) formatted certificate or certificate chain."""
    CER = "Cer"
    """The certificate is a base64-encoded X.509 certificate."""


class CertificateProvisioningState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """CertificateProvisioningState."""

    SUCCEEDED = "Succeeded"
    """The certificate is available for use in pools."""
    DELETING = "Deleting"
    """The user has requested that the certificate be deleted, but the delete operation has not yet
    #: completed. You may not reference the certificate when creating or updating pools."""
    FAILED = "Failed"
    """The user requested that the certificate be deleted, but there are pools that still have
    #: references to the certificate, or it is still installed on one or more compute nodes. (The
    #: latter can occur if the certificate has been removed from the pool, but the node has not yet
    #: restarted. Nodes refresh their certificates only when they restart.) You may use the cancel
    #: certificate delete operation to cancel the delete, or the delete certificate operation to retry
    #: the delete."""


class CertificateStoreLocation(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The default value is currentUser. This property is applicable only for pools configured with
    Windows nodes (that is, created with cloudServiceConfiguration, or with
    virtualMachineConfiguration using a Windows image reference). For Linux compute nodes, the
    certificates are stored in a directory inside the task working directory and an environment
    variable AZ_BATCH_CERTIFICATES_DIR is supplied to the task to query for this location. For
    certificates with visibility of 'remoteUser', a 'certs' directory is created in the user's home
    directory (e.g., /home/{user-name}/certs) and certificates are placed in that directory.
    """

    CURRENT_USER = "CurrentUser"
    """Certificates should be installed to the CurrentUser certificate store."""
    LOCAL_MACHINE = "LocalMachine"
    """Certificates should be installed to the LocalMachine certificate store."""


class CertificateVisibility(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """CertificateVisibility."""

    START_TASK = "StartTask"
    """The certificate should be visible to the user account under which the start task is run. Note
    #: that if AutoUser Scope is Pool for both the StartTask and a Task, this certificate will be
    #: visible to the Task as well."""
    TASK = "Task"
    """The certificate should be visible to the user accounts under which job tasks are run."""
    REMOTE_USER = "RemoteUser"
    """The certificate should be visible to the user accounts under which users remotely access the
    #: node."""


class ComputeNodeDeallocationOption(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Determines what to do with a node and its running task(s) after it has been selected for
    deallocation.
    """

    REQUEUE = "Requeue"
    """Terminate running task processes and requeue the tasks. The tasks will run again when a node is
    #: available. Remove nodes as soon as tasks have been terminated."""
    TERMINATE = "Terminate"
    """Terminate running tasks. The tasks will be completed with failureInfo indicating that they were
    #: terminated, and will not run again. Remove nodes as soon as tasks have been terminated."""
    TASK_COMPLETION = "TaskCompletion"
    """Allow currently running tasks to complete. Schedule no new tasks while waiting. Remove nodes
    #: when all tasks have completed."""
    RETAINED_DATA = "RetainedData"
    """Allow currently running tasks to complete, then wait for all task data retention periods to
    #: expire. Schedule no new tasks while waiting. Remove nodes when all task retention periods have
    #: expired."""


class ComputeNodeFillType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """How tasks should be distributed across compute nodes."""

    SPREAD = "Spread"
    """Tasks should be assigned evenly across all nodes in the pool."""
    PACK = "Pack"
    """As many tasks as possible (taskSlotsPerNode) should be assigned to each node in the pool before
    #: any tasks are assigned to the next node in the pool."""


class ContainerType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The container technology to be used."""

    DOCKER_COMPATIBLE = "DockerCompatible"
    """A Docker compatible container technology will be used to launch the containers."""
    CRI_COMPATIBLE = "CriCompatible"
    """A CRI based technology will be used to launch the containers."""


class ContainerWorkingDirectory(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """A flag to indicate where the container task working directory is. The default is
    'taskWorkingDirectory'.
    """

    TASK_WORKING_DIRECTORY = "TaskWorkingDirectory"
    """Use the standard Batch service task working directory, which will contain the Task resource
    #: files populated by Batch."""
    CONTAINER_IMAGE_DEFAULT = "ContainerImageDefault"
    """Using container image defined working directory. Beware that this directory will not contain
    #: the resource files downloaded by Batch."""


class DiskEncryptionTarget(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """If omitted, no disks on the compute nodes in the pool will be encrypted."""

    OS_DISK = "OsDisk"
    """The OS Disk on the compute node is encrypted."""
    TEMPORARY_DISK = "TemporaryDisk"
    """The temporary disk on the compute node is encrypted. On Linux this encryption applies to other
    #: partitions (such as those on mounted data disks) when encryption occurs at boot time."""


class DynamicVNetAssignmentScope(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The scope of dynamic vnet assignment."""

    NONE = "none"
    """No dynamic VNet assignment is enabled."""
    JOB = "job"
    """Dynamic VNet assignment is done per-job. If this value is set, the network configuration subnet
    #: ID must also be set. This feature requires approval before use, please contact support"""


class ElevationLevel(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The elevation level of the user."""

    NON_ADMIN = "NonAdmin"
    """The user is a standard user without elevated access."""
    ADMIN = "Admin"
    """The user is a user with elevated access and operates with full Administrator permissions."""


class EndpointAccessDefaultAction(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Default action for endpoint access. It is only applicable when publicNetworkAccess is enabled."""

    ALLOW = "Allow"
    """Allow client access."""
    DENY = "Deny"
    """Deny client access."""


class InboundEndpointProtocol(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The protocol of the endpoint."""

    TCP = "TCP"
    """Use TCP for the endpoint."""
    UDP = "UDP"
    """Use UDP for the endpoint."""


class InterNodeCommunicationState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """This imposes restrictions on which nodes can be assigned to the pool. Enabling this value can
    reduce the chance of the requested number of nodes to be allocated in the pool. If not
    specified, this value defaults to 'Disabled'.
    """

    ENABLED = "Enabled"
    """Enable network communication between virtual machines."""
    DISABLED = "Disabled"
    """Disable network communication between virtual machines."""


class IPAddressProvisioningType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The provisioning type for Public IP Addresses for the Batch Pool."""

    BATCH_MANAGED = "BatchManaged"
    """A public IP will be created and managed by Batch. There may be multiple public IPs depending on
    #: the size of the Pool."""
    USER_MANAGED = "UserManaged"
    """Public IPs are provided by the user and will be used to provision the Compute Nodes."""
    NO_PUBLIC_IP_ADDRESSES = "NoPublicIPAddresses"
    """No public IP Address will be created for the Compute Nodes in the Pool."""


class KeySource(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of the key source."""

    MICROSOFT_BATCH = "Microsoft.Batch"
    """Batch creates and manages the encryption keys used to protect the account data."""
    MICROSOFT_KEY_VAULT = "Microsoft.KeyVault"
    """The encryption keys used to protect the account data are stored in an external key vault. If
    #: this is set then the Batch Account identity must be set to ``SystemAssigned`` and a valid Key
    #: Identifier must also be supplied under the keyVaultProperties."""


class LoginMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies login mode for the user. The default value for VirtualMachineConfiguration pools is
    interactive mode and for CloudServiceConfiguration pools is batch mode.
    """

    BATCH = "Batch"
    """The LOGON32_LOGON_BATCH Win32 login mode. The batch login mode is recommended for long running
    #: parallel processes."""
    INTERACTIVE = "Interactive"
    """The LOGON32_LOGON_INTERACTIVE Win32 login mode. Some applications require having permissions
    #: associated with the interactive login mode. If this is the case for an application used in your
    #: task, then this option is recommended."""


class NameAvailabilityReason(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Gets the reason that a Batch account name could not be used. The Reason element is only
    returned if NameAvailable is false.
    """

    INVALID = "Invalid"
    """The requested name is invalid."""
    ALREADY_EXISTS = "AlreadyExists"
    """The requested name is already in use."""


class NetworkSecurityGroupRuleAccess(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The action that should be taken for a specified IP address, subnet range or tag."""

    ALLOW = "Allow"
    """Allow access."""
    DENY = "Deny"
    """Deny access."""


class NodeCommunicationMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Determines how a pool communicates with the Batch service."""

    DEFAULT = "Default"
    """The node communication mode is automatically set by the Batch service."""
    CLASSIC = "Classic"
    """Nodes using the Classic communication mode require inbound TCP communication on ports 29876 and
    #: 29877 from the "BatchNodeManagement.{region}" service tag and outbound TCP communication on
    #: port 443 to the "Storage.region" and "BatchNodeManagement.{region}" service tags."""
    SIMPLIFIED = "Simplified"
    """Nodes using the Simplified communication mode require outbound TCP communication on port 443 to
    #: the "BatchNodeManagement.{region}" service tag. No open inbound ports are required."""


class NodePlacementPolicyType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The default value is regional."""

    REGIONAL = "Regional"
    """All nodes in the pool will be allocated in the same region."""
    ZONAL = "Zonal"
    """Nodes in the pool will be spread across different zones with best effort balancing."""


class PackageState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The current state of the application package."""

    PENDING = "Pending"
    """The application package has been created but has not yet been activated."""
    ACTIVE = "Active"
    """The application package is ready for use."""


class PoolAllocationMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The allocation mode for creating pools in the Batch account."""

    BATCH_SERVICE = "BatchService"
    """Pools will be allocated in subscriptions owned by the Batch service."""
    USER_SUBSCRIPTION = "UserSubscription"
    """Pools will be allocated in a subscription owned by the user."""


class PoolIdentityType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of identity used for the Batch Pool."""

    USER_ASSIGNED = "UserAssigned"
    """Batch pool has user assigned identities with it."""
    NONE = "None"
    """Batch pool has no identity associated with it. Setting ``None`` in update pool will remove
    #: existing identities."""


class PoolProvisioningState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The current state of the pool."""

    SUCCEEDED = "Succeeded"
    """The pool is available to run tasks subject to the availability of compute nodes."""
    DELETING = "Deleting"
    """The user has requested that the pool be deleted, but the delete operation has not yet
    #: completed."""


class PrivateEndpointConnectionProvisioningState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The provisioning state of the private endpoint connection."""

    CREATING = "Creating"
    """The connection is creating."""
    UPDATING = "Updating"
    """The user has requested that the connection status be updated, but the update operation has not
    #: yet completed. You may not reference the connection when connecting the Batch account."""
    DELETING = "Deleting"
    """The connection is deleting."""
    SUCCEEDED = "Succeeded"
    """The connection status is final and is ready for use if Status is Approved."""
    FAILED = "Failed"
    """The user requested that the connection be updated and it failed. You may retry the update
    #: operation."""
    CANCELLED = "Cancelled"
    """The user has cancelled the connection creation."""


class PrivateLinkServiceConnectionStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The status of the Batch private endpoint connection."""

    APPROVED = "Approved"
    """The private endpoint connection is approved and can be used to access Batch account"""
    PENDING = "Pending"
    """The private endpoint connection is pending and cannot be used to access Batch account"""
    REJECTED = "Rejected"
    """The private endpoint connection is rejected and cannot be used to access Batch account"""
    DISCONNECTED = "Disconnected"
    """The private endpoint connection is disconnected and cannot be used to access Batch account"""


class ProvisioningState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The provisioned state of the resource."""

    INVALID = "Invalid"
    """The account is in an invalid state."""
    CREATING = "Creating"
    """The account is being created."""
    DELETING = "Deleting"
    """The account is being deleted."""
    SUCCEEDED = "Succeeded"
    """The account has been created and is ready for use."""
    FAILED = "Failed"
    """The last operation for the account is failed."""
    CANCELLED = "Cancelled"
    """The last operation for the account is cancelled."""


class PublicNetworkAccessType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The network access type for operating on the resources in the Batch account."""

    ENABLED = "Enabled"
    """Enables connectivity to Azure Batch through public DNS."""
    DISABLED = "Disabled"
    """Disables public connectivity and enables private connectivity to Azure Batch Service through
    #: private endpoint resource."""


class ResourceIdentityType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of identity used for the Batch account."""

    SYSTEM_ASSIGNED = "SystemAssigned"
    """Batch account has a system assigned identity with it."""
    USER_ASSIGNED = "UserAssigned"
    """Batch account has user assigned identities with it."""
    NONE = "None"
    """Batch account has no identity associated with it. Setting ``None`` in update account will
    #: remove existing identities."""


class StorageAccountType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The storage account type for use in creating data disks or OS disk."""

    STANDARD_LRS = "Standard_LRS"
    """The data disk / OS disk should use standard locally redundant storage."""
    PREMIUM_LRS = "Premium_LRS"
    """The data disk / OS disk should use premium locally redundant storage."""
    STANDARD_SSD_LRS = "StandardSSD_LRS"
    """The data disk / OS disk should use standard SSD locally redundant storage."""


class UpgradeMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the mode of an upgrade to virtual machines in the scale set.:code:`<br />`:code:`<br
    />` Possible values are::code:`<br />`:code:`<br />` **Manual** - You  control the application
    of updates to virtual machines in the scale set. You do this by using the manualUpgrade
    action.:code:`<br />`:code:`<br />` **Automatic** - All virtual machines in the scale set are
    automatically updated at the same time.:code:`<br />`:code:`<br />` **Rolling** - Scale set
    performs updates in batches with an optional pause time in between.
    """

    AUTOMATIC = "automatic"
    """All virtual machines in the scale set are automatically updated at the same time."""
    MANUAL = "manual"
    """You control the application of updates to virtual machines in the scale set. You do this by
    #: using the manualUpgrade action."""
    ROLLING = "rolling"
    """The existing instances in a scale set are brought down in batches to be upgraded. Once the
    #: upgraded batch is complete, the instances will begin taking traffic again and the next batch
    #: will begin. This continues until all instances brought up-to-date."""



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import Any, Optional, TYPE_CHECKING, cast
from typing_extensions import Self

from azure.core.pipeline import policies
from azure.core.rest import HttpRequest, HttpResponse
from azure.core.settings import settings
from azure.mgmt.core import ARMPipelineClient
from azure.mgmt.core.policies import ARMAutoResourceProviderRegistrationPolicy
from azure.mgmt.core.tools import get_arm_endpoints

from . import models as _models
from ._configuration import AuthorizationManagementClientConfiguration
from ._utils.serialization import Deserializer, Serializer
from .operations import (
    AccessReviewDefaultSettingsOperations,
    AccessReviewHistoryDefinitionInstanceOperations,
    AccessReviewHistoryDefinitionInstancesOperations,
    AccessReviewHistoryDefinitionOperations,
    AccessReviewHistoryDefinitionsOperations,
    AccessReviewInstanceContactedReviewersOperations,
    AccessReviewInstanceDecisionsOperations,
    AccessReviewInstanceMyDecisionsOperations,
    AccessReviewInstanceOperations,
    AccessReviewInstancesAssignedForMyApprovalOperations,
    AccessReviewInstancesOperations,
    AccessReviewScheduleDefinitionsAssignedForMyApprovalOperations,
    AccessReviewScheduleDefinitionsOperations,
    AlertConfigurationsOperations,
    AlertDefinitionsOperations,
    AlertIncidentsOperations,
    AlertOperationOperations,
    AlertsOperations,
    ClassicAdministratorsOperations,
    DenyAssignmentsOperations,
    EligibleChildResourcesOperations,
    GlobalAdministratorOperations,
    Operations,
    PermissionsOperations,
    ProviderOperationsMetadataOperations,
    RoleAssignmentScheduleInstancesOperations,
    RoleAssignmentScheduleRequestsOperations,
    RoleAssignmentSchedulesOperations,
    RoleAssignmentsOperations,
    RoleDefinitionsOperations,
    RoleEligibilityScheduleInstancesOperations,
    RoleEligibilityScheduleRequestsOperations,
    RoleEligibilitySchedulesOperations,
    RoleManagementPoliciesOperations,
    RoleManagementPolicyAssignmentsOperations,
    ScopeAccessReviewDefaultSettingsOperations,
    ScopeAccessReviewHistoryDefinitionInstanceOperations,
    ScopeAccessReviewHistoryDefinitionInstancesOperations,
    ScopeAccessReviewHistoryDefinitionOperations,
    ScopeAccessReviewHistoryDefinitionsOperations,
    ScopeAccessReviewInstanceContactedReviewersOperations,
    ScopeAccessReviewInstanceDecisionsOperations,
    ScopeAccessReviewInstanceOperations,
    ScopeAccessReviewInstancesOperations,
    ScopeAccessReviewScheduleDefinitionsOperations,
    TenantLevelAccessReviewInstanceContactedReviewersOperations,
)

if TYPE_CHECKING:
    from azure.core.credentials import TokenCredential


class AuthorizationManagementClient:  # pylint: disable=client-accepts-api-version-keyword,too-many-instance-attributes
    """Role based access control provides you a way to apply granular level policy administration down
    to individual resources or resource groups. These operations enable you to manage role
    definitions and role assignments. A role definition describes the set of actions that can be
    performed on resources. A role assignment grants access to Azure Active Directory users.

    :ivar classic_administrators: ClassicAdministratorsOperations operations
    :vartype classic_administrators:
     azure.mgmt.authorization.operations.ClassicAdministratorsOperations
    :ivar global_administrator: GlobalAdministratorOperations operations
    :vartype global_administrator:
     azure.mgmt.authorization.operations.GlobalAdministratorOperations
    :ivar deny_assignments: DenyAssignmentsOperations operations
    :vartype deny_assignments: azure.mgmt.authorization.operations.DenyAssignmentsOperations
    :ivar provider_operations_metadata: ProviderOperationsMetadataOperations operations
    :vartype provider_operations_metadata:
     azure.mgmt.authorization.operations.ProviderOperationsMetadataOperations
    :ivar role_assignments: RoleAssignmentsOperations operations
    :vartype role_assignments: azure.mgmt.authorization.operations.RoleAssignmentsOperations
    :ivar permissions: PermissionsOperations operations
    :vartype permissions: azure.mgmt.authorization.operations.PermissionsOperations
    :ivar role_definitions: RoleDefinitionsOperations operations
    :vartype role_definitions: azure.mgmt.authorization.operations.RoleDefinitionsOperations
    :ivar operations: Operations operations
    :vartype operations: azure.mgmt.authorization.operations.Operations
    :ivar access_review_history_definitions: AccessReviewHistoryDefinitionsOperations operations
    :vartype access_review_history_definitions:
     azure.mgmt.authorization.operations.AccessReviewHistoryDefinitionsOperations
    :ivar access_review_history_definition: AccessReviewHistoryDefinitionOperations operations
    :vartype access_review_history_definition:
     azure.mgmt.authorization.operations.AccessReviewHistoryDefinitionOperations
    :ivar access_review_history_definition_instance:
     AccessReviewHistoryDefinitionInstanceOperations operations
    :vartype access_review_history_definition_instance:
     azure.mgmt.authorization.operations.AccessReviewHistoryDefinitionInstanceOperations
    :ivar access_review_history_definition_instances:
     AccessReviewHistoryDefinitionInstancesOperations operations
    :vartype access_review_history_definition_instances:
     azure.mgmt.authorization.operations.AccessReviewHistoryDefinitionInstancesOperations
    :ivar access_review_schedule_definitions: AccessReviewScheduleDefinitionsOperations operations
    :vartype access_review_schedule_definitions:
     azure.mgmt.authorization.operations.AccessReviewScheduleDefinitionsOperations
    :ivar access_review_instances: AccessReviewInstancesOperations operations
    :vartype access_review_instances:
     azure.mgmt.authorization.operations.AccessReviewInstancesOperations
    :ivar access_review_instance: AccessReviewInstanceOperations operations
    :vartype access_review_instance:
     azure.mgmt.authorization.operations.AccessReviewInstanceOperations
    :ivar access_review_instance_decisions: AccessReviewInstanceDecisionsOperations operations
    :vartype access_review_instance_decisions:
     azure.mgmt.authorization.operations.AccessReviewInstanceDecisionsOperations
    :ivar access_review_instance_contacted_reviewers:
     AccessReviewInstanceContactedReviewersOperations operations
    :vartype access_review_instance_contacted_reviewers:
     azure.mgmt.authorization.operations.AccessReviewInstanceContactedReviewersOperations
    :ivar access_review_default_settings: AccessReviewDefaultSettingsOperations operations
    :vartype access_review_default_settings:
     azure.mgmt.authorization.operations.AccessReviewDefaultSettingsOperations
    :ivar scope_access_review_history_definitions: ScopeAccessReviewHistoryDefinitionsOperations
     operations
    :vartype scope_access_review_history_definitions:
     azure.mgmt.authorization.operations.ScopeAccessReviewHistoryDefinitionsOperations
    :ivar scope_access_review_history_definition: ScopeAccessReviewHistoryDefinitionOperations
     operations
    :vartype scope_access_review_history_definition:
     azure.mgmt.authorization.operations.ScopeAccessReviewHistoryDefinitionOperations
    :ivar scope_access_review_history_definition_instance:
     ScopeAccessReviewHistoryDefinitionInstanceOperations operations
    :vartype scope_access_review_history_definition_instance:
     azure.mgmt.authorization.operations.ScopeAccessReviewHistoryDefinitionInstanceOperations
    :ivar scope_access_review_history_definition_instances:
     ScopeAccessReviewHistoryDefinitionInstancesOperations operations
    :vartype scope_access_review_history_definition_instances:
     azure.mgmt.authorization.operations.ScopeAccessReviewHistoryDefinitionInstancesOperations
    :ivar scope_access_review_schedule_definitions: ScopeAccessReviewScheduleDefinitionsOperations
     operations
    :vartype scope_access_review_schedule_definitions:
     azure.mgmt.authorization.operations.ScopeAccessReviewScheduleDefinitionsOperations
    :ivar scope_access_review_instances: ScopeAccessReviewInstancesOperations operations
    :vartype scope_access_review_instances:
     azure.mgmt.authorization.operations.ScopeAccessReviewInstancesOperations
    :ivar scope_access_review_instance: ScopeAccessReviewInstanceOperations operations
    :vartype scope_access_review_instance:
     azure.mgmt.authorization.operations.ScopeAccessReviewInstanceOperations
    :ivar scope_access_review_instance_decisions: ScopeAccessReviewInstanceDecisionsOperations
     operations
    :vartype scope_access_review_instance_decisions:
     azure.mgmt.authorization.operations.ScopeAccessReviewInstanceDecisionsOperations
    :ivar scope_access_review_instance_contacted_reviewers:
     ScopeAccessReviewInstanceContactedReviewersOperations operations
    :vartype scope_access_review_instance_contacted_reviewers:
     azure.mgmt.authorization.operations.ScopeAccessReviewInstanceContactedReviewersOperations
    :ivar scope_access_review_default_settings: ScopeAccessReviewDefaultSettingsOperations
     operations
    :vartype scope_access_review_default_settings:
     azure.mgmt.authorization.operations.ScopeAccessReviewDefaultSettingsOperations
    :ivar access_review_schedule_definitions_assigned_for_my_approval:
     AccessReviewScheduleDefinitionsAssignedForMyApprovalOperations operations
    :vartype access_review_schedule_definitions_assigned_for_my_approval:
     azure.mgmt.authorization.operations.AccessReviewScheduleDefinitionsAssignedForMyApprovalOperations
    :ivar access_review_instances_assigned_for_my_approval:
     AccessReviewInstancesAssignedForMyApprovalOperations operations
    :vartype access_review_instances_assigned_for_my_approval:
     azure.mgmt.authorization.operations.AccessReviewInstancesAssignedForMyApprovalOperations
    :ivar access_review_instance_my_decisions: AccessReviewInstanceMyDecisionsOperations operations
    :vartype access_review_instance_my_decisions:
     azure.mgmt.authorization.operations.AccessReviewInstanceMyDecisionsOperations
    :ivar tenant_level_access_review_instance_contacted_reviewers:
     TenantLevelAccessReviewInstanceContactedReviewersOperations operations
    :vartype tenant_level_access_review_instance_contacted_reviewers:
     azure.mgmt.authorization.operations.TenantLevelAccessReviewInstanceContactedReviewersOperations
    :ivar eligible_child_resources: EligibleChildResourcesOperations operations
    :vartype eligible_child_resources:
     azure.mgmt.authorization.operations.EligibleChildResourcesOperations
    :ivar role_assignment_schedules: RoleAssignmentSchedulesOperations operations
    :vartype role_assignment_schedules:
     azure.mgmt.authorization.operations.RoleAssignmentSchedulesOperations
    :ivar role_assignment_schedule_instances: RoleAssignmentScheduleInstancesOperations operations
    :vartype role_assignment_schedule_instances:
     azure.mgmt.authorization.operations.RoleAssignmentScheduleInstancesOperations
    :ivar role_assignment_schedule_requests: RoleAssignmentScheduleRequestsOperations operations
    :vartype role_assignment_schedule_requests:
     azure.mgmt.authorization.operations.RoleAssignmentScheduleRequestsOperations
    :ivar role_eligibility_schedules: RoleEligibilitySchedulesOperations operations
    :vartype role_eligibility_schedules:
     azure.mgmt.authorization.operations.RoleEligibilitySchedulesOperations
    :ivar role_eligibility_schedule_instances: RoleEligibilityScheduleInstancesOperations
     operations
    :vartype role_eligibility_schedule_instances:
     azure.mgmt.authorization.operations.RoleEligibilityScheduleInstancesOperations
    :ivar role_eligibility_schedule_requests: RoleEligibilityScheduleRequestsOperations operations
    :vartype role_eligibility_schedule_requests:
     azure.mgmt.authorization.operations.RoleEligibilityScheduleRequestsOperations
    :ivar role_management_policies: RoleManagementPoliciesOperations operations
    :vartype role_management_policies:
     azure.mgmt.authorization.operations.RoleManagementPoliciesOperations
    :ivar role_management_policy_assignments: RoleManagementPolicyAssignmentsOperations operations
    :vartype role_management_policy_assignments:
     azure.mgmt.authorization.operations.RoleManagementPolicyAssignmentsOperations
    :ivar alerts: AlertsOperations operations
    :vartype alerts: azure.mgmt.authorization.operations.AlertsOperations
    :ivar alert_configurations: AlertConfigurationsOperations operations
    :vartype alert_configurations:
     azure.mgmt.authorization.operations.AlertConfigurationsOperations
    :ivar alert_definitions: AlertDefinitionsOperations operations
    :vartype alert_definitions: azure.mgmt.authorization.operations.AlertDefinitionsOperations
    :ivar alert_incidents: AlertIncidentsOperations operations
    :vartype alert_incidents: azure.mgmt.authorization.operations.AlertIncidentsOperations
    :ivar alert_operation: AlertOperationOperations operations
    :vartype alert_operation: azure.mgmt.authorization.operations.AlertOperationOperations
    :param credential: Credential needed for the client to connect to Azure. Required.
    :type credential: ~azure.core.credentials.TokenCredential
    :param subscription_id: The ID of the target subscription. Required.
    :type subscription_id: str
    :param base_url: Service URL. Default value is None.
    :type base_url: str
    :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
     Retry-After header is present.
    """

    def __init__(
        self, credential: "TokenCredential", subscription_id: str, base_url: Optional[str] = None, **kwargs: Any
    ) -> None:
        _cloud = kwargs.pop("cloud_setting", None) or settings.current.azure_cloud  # type: ignore
        _endpoints = get_arm_endpoints(_cloud)
        if not base_url:
            base_url = _endpoints["resource_manager"]
        credential_scopes = kwargs.pop("credential_scopes", _endpoints["credential_scopes"])
        self._config = AuthorizationManagementClientConfiguration(
            credential=credential, subscription_id=subscription_id, credential_scopes=credential_scopes, **kwargs
        )

        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                policies.RequestIdPolicy(**kwargs),
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                ARMAutoResourceProviderRegistrationPolicy(),
                self._config.redirect_policy,
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.custom_hook_policy,
                self._config.logging_policy,
                policies.DistributedTracingPolicy(**kwargs),
                policies.SensitiveHeaderCleanupPolicy(**kwargs) if self._config.redirect_policy else None,
                self._config.http_logging_policy,
            ]
        self._client: ARMPipelineClient = ARMPipelineClient(base_url=cast(str, base_url), policies=_policies, **kwargs)

        client_models = {k: v for k, v in _models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)
        self._serialize.client_side_validation = False
        self.classic_administrators = ClassicAdministratorsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.global_administrator = GlobalAdministratorOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.deny_assignments = DenyAssignmentsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.provider_operations_metadata = ProviderOperationsMetadataOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.role_assignments = RoleAssignmentsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.permissions = PermissionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.role_definitions = RoleDefinitionsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.operations = Operations(self._client, self._config, self._serialize, self._deserialize)
        self.access_review_history_definitions = AccessReviewHistoryDefinitionsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.access_review_history_definition = AccessReviewHistoryDefinitionOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.access_review_history_definition_instance = AccessReviewHistoryDefinitionInstanceOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.access_review_history_definition_instances = AccessReviewHistoryDefinitionInstancesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.access_review_schedule_definitions = AccessReviewScheduleDefinitionsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.access_review_instances = AccessReviewInstancesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.access_review_instance = AccessReviewInstanceOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.access_review_instance_decisions = AccessReviewInstanceDecisionsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.access_review_instance_contacted_reviewers = AccessReviewInstanceContactedReviewersOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.access_review_default_settings = AccessReviewDefaultSettingsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.scope_access_review_history_definitions = ScopeAccessReviewHistoryDefinitionsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.scope_access_review_history_definition = ScopeAccessReviewHistoryDefinitionOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.scope_access_review_history_definition_instance = ScopeAccessReviewHistoryDefinitionInstanceOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.scope_access_review_history_definition_instances = ScopeAccessReviewHistoryDefinitionInstancesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.scope_access_review_schedule_definitions = ScopeAccessReviewScheduleDefinitionsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.scope_access_review_instances = ScopeAccessReviewInstancesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.scope_access_review_instance = ScopeAccessReviewInstanceOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.scope_access_review_instance_decisions = ScopeAccessReviewInstanceDecisionsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.scope_access_review_instance_contacted_reviewers = ScopeAccessReviewInstanceContactedReviewersOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.scope_access_review_default_settings = ScopeAccessReviewDefaultSettingsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.access_review_schedule_definitions_assigned_for_my_approval = (
            AccessReviewScheduleDefinitionsAssignedForMyApprovalOperations(
                self._client, self._config, self._serialize, self._deserialize
            )
        )
        self.access_review_instances_assigned_for_my_approval = AccessReviewInstancesAssignedForMyApprovalOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.access_review_instance_my_decisions = AccessReviewInstanceMyDecisionsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.tenant_level_access_review_instance_contacted_reviewers = (
            TenantLevelAccessReviewInstanceContactedReviewersOperations(
                self._client, self._config, self._serialize, self._deserialize
            )
        )
        self.eligible_child_resources = EligibleChildResourcesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.role_assignment_schedules = RoleAssignmentSchedulesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.role_assignment_schedule_instances = RoleAssignmentScheduleInstancesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.role_assignment_schedule_requests = RoleAssignmentScheduleRequestsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.role_eligibility_schedules = RoleEligibilitySchedulesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.role_eligibility_schedule_instances = RoleEligibilityScheduleInstancesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.role_eligibility_schedule_requests = RoleEligibilityScheduleRequestsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.role_management_policies = RoleManagementPoliciesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.role_management_policy_assignments = RoleManagementPolicyAssignmentsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.alerts = AlertsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.alert_configurations = AlertConfigurationsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.alert_definitions = AlertDefinitionsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.alert_incidents = AlertIncidentsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.alert_operation = AlertOperationOperations(self._client, self._config, self._serialize, self._deserialize)

    def _send_request(self, request: HttpRequest, *, stream: bool = False, **kwargs: Any) -> HttpResponse:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = client._send_request(request)
        <HttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.HttpResponse
        """

        request_copy = deepcopy(request)
        request_copy.url = self._client.format_url(request_copy.url)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    def close(self) -> None:
        self._client.close()

    def __enter__(self) -> Self:
        self._client.__enter__()
        return self

    def __exit__(self, *exc_details: Any) -> None:
        self._client.__exit__(*exc_details)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=no-self-use, line-too-long, protected-access, too-few-public-methods, unused-argument
from knack.log import get_logger

from ..aaz.latest.vm.host import Show as _VMHostShow

logger = get_logger(__name__)


class VMHostShow(_VMHostShow):
    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        if 'tags' not in result:
            result['tags'] = {}
        return result


def convert_show_result_to_snake_case(result):
    new_result = {}
    if 'location' in result:
        new_result['location'] = result['location']

    if 'sku' in result:
        new_result['sku'] = result['sku']

    if 'tags' in result:
        new_result['tags'] = result['tags']

    if 'autoReplaceOnFailure' in result:
        new_result['auto_replace_on_failure'] = result['autoReplaceOnFailure']

    if 'licenseType' in result:
        new_result['license_type'] = result['licenseType']

    if 'platformFaultDomain' in result:
        new_result['platform_fault_domain'] = result['platformFaultDomain']

    return new_result



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import random
import re
import semver
from typing import Any, Dict, List, TypeVar

from azure.cli.command_modules.acs._client_factory import get_snapshots_client, get_msi_client
from azure.cli.core.azclierror import (
    AzureInternalError,
    AzureResponseError,
    BadRequestError,
    ClientRequestError,
    ForbiddenError,
    InvalidArgumentValueError,
    ResourceNotFoundError,
    ServiceError,
    UnauthorizedError,
    UnclassifiedUserFault,
)
from azure.core.exceptions import AzureError, HttpResponseError, ServiceRequestError, ServiceResponseError

# type variables
ManagedCluster = TypeVar("ManagedCluster")


def get_monitoring_addon_key(addon_profiles, monitoring_addon_name):
    """Return the canonical key for the monitoring addon, normalizing non-standard casing.

    The API response may return the monitoring addon key in any casing (e.g.
    "omsagent", "omsAgent", "oMSaGent").  This helper performs a
    case-insensitive lookup and, when a non-standard key is found, re-keys
    addon_profiles in-place so that subsequent code always uses the canonical
    ``monitoring_addon_name`` (lowercase) form.
    """
    if addon_profiles is None:
        return monitoring_addon_name
    # Exact match on the canonical lowercase name – preferred form.
    if monitoring_addon_name in addon_profiles:
        return monitoring_addon_name
    # Case-insensitive fallback: catch any casing the server may return.
    target_lower = monitoring_addon_name.lower()
    for key in list(addon_profiles):
        if key.lower() == target_lower:
            # Normalize: move the profile to the canonical key.
            addon_profiles[monitoring_addon_name] = addon_profiles.pop(key)
            return monitoring_addon_name
    return monitoring_addon_name


def format_parameter_name_to_option_name(parameter_name: str) -> str:
    """Convert a name in parameter format to option format.

    Underscores ("_") are used to connect the various parts of a parameter name, while hyphens ("-") are used to connect
    each part of an option name. Besides, the option name starts with double hyphens ("--").

    :return: str
    """
    option_name = "--" + parameter_name.replace("_", "-")
    return option_name


def safe_list_get(li: List, idx: int, default: Any = None) -> Any:
    """Get an element from a list without raising IndexError.

    Attempt to get the element with index idx from a list-like object li, and if the index is invalid (such as out of
    range), return default (whose default value is None).

    :return: an element of any type
    """
    if isinstance(li, list):
        try:
            return li[idx]
        except IndexError:
            return default
    return None


def safe_lower(obj: Any) -> Any:
    """Return lowercase string if the provided obj is a string, otherwise return the object itself.

    :return: Any
    """
    if isinstance(obj, str):
        return obj.lower()
    return obj


def build_etag_kwargs(if_match=None, if_none_match=None) -> Dict[str, Any]:
    """Convert if_match/if_none_match to etag/match_condition kwargs for SDK v41+."""
    from azure.core import MatchConditions
    kwargs: Dict[str, Any] = {}
    if if_match is not None:
        kwargs["etag"] = if_match
        kwargs["match_condition"] = MatchConditions.IfNotModified
    elif if_none_match is not None:
        if if_none_match == "*":
            kwargs["match_condition"] = MatchConditions.IfMissing
        else:
            kwargs["etag"] = if_none_match
            kwargs["match_condition"] = MatchConditions.IfModified
    return kwargs


def get_property_from_dict_or_object(obj, property_name) -> Any:
    """Get the value corresponding to the property name from a dictionary or object.

    Note: Would raise exception if the property does not exist.

    :return: Any
    """
    if isinstance(obj, dict):
        return obj[property_name]
    return getattr(obj, property_name)


def check_is_msi_cluster(mc: ManagedCluster) -> bool:
    """Check `mc` object to determine whether managed identity is enabled.

    :return: bool
    """
    if mc and mc.identity and mc.identity.type is not None:
        identity_type = mc.identity.type.casefold()
        if identity_type in ("systemassigned", "userassigned"):
            return True
    return False


def check_is_private_cluster(mc: ManagedCluster) -> bool:
    """Check `mc` object to determine whether private cluster is enabled.

    :return: bool
    """
    if mc and mc.api_server_access_profile:
        return bool(mc.api_server_access_profile.enable_private_cluster)
    return False


def check_is_apiserver_vnet_integration_cluster(mc: ManagedCluster) -> bool:
    """Check `mc` object to determine whether apiserver vnet integration is enabled.

    :return: bool
    """
    if mc and mc.api_server_access_profile:
        return bool(mc.api_server_access_profile.enable_vnet_integration)
    return False


def check_is_private_link_cluster(mc: ManagedCluster) -> bool:
    """Check `mc` object to determine whether private link cluster is enabled.
    :return: bool
    """
    return check_is_private_cluster(mc) and not check_is_apiserver_vnet_integration_cluster(mc)


def check_is_managed_aad_cluster(mc: ManagedCluster) -> bool:
    """Check `mc` object to determine whether managed aad is enabled.

    :return: bool
    """
    if mc and mc.aad_profile is not None and mc.aad_profile.managed:
        return True
    return False


# pylint: disable=too-many-return-statements
def map_azure_error_to_cli_error(azure_error):
    error_message = getattr(azure_error, "message", str(azure_error))
    if isinstance(azure_error, HttpResponseError):
        status_code = getattr(azure_error, "status_code", None)
        if status_code:
            status_code = int(status_code)
            if status_code == 400:
                return BadRequestError(error_message)
            if status_code == 401:
                return UnauthorizedError(error_message)
            if status_code == 403:
                return ForbiddenError(error_message)
            if status_code == 404:
                return ResourceNotFoundError(error_message)
            if 400 <= status_code < 500:
                return UnclassifiedUserFault(error_message)
            if 500 <= status_code < 600:
                return AzureInternalError(error_message)
        return ServiceError(error_message)
    if isinstance(azure_error, ServiceRequestError):
        return ClientRequestError(error_message)
    if isinstance(azure_error, ServiceResponseError):
        return AzureResponseError(error_message)
    return ServiceError(error_message)


def get_snapshot_by_snapshot_id(cli_ctx, snapshot_id):
    _re_snapshot_resource_id = re.compile(
        r"/subscriptions/(.*?)/resourcegroups/(.*?)/providers/microsoft.containerservice/snapshots/(.*)",
        flags=re.IGNORECASE,
    )
    snapshot_id = snapshot_id.lower()
    match = _re_snapshot_resource_id.search(snapshot_id)
    if match:
        subscription_id = match.group(1)
        resource_group_name = match.group(2)
        snapshot_name = match.group(3)
        return get_snapshot(cli_ctx, subscription_id, resource_group_name, snapshot_name)
    raise InvalidArgumentValueError("Cannot parse snapshot name from provided resource id '{}'.".format(snapshot_id))


def get_snapshot(cli_ctx, subscription_id, resource_group_name, snapshot_name):
    snapshot_client = get_snapshots_client(cli_ctx, subscription_id=subscription_id)
    try:
        snapshot = snapshot_client.get(resource_group_name, snapshot_name)
    # track 2 sdk raise exception from azure.core.exceptions
    except AzureError as ex:
        if "not found" in ex.message:
            raise ResourceNotFoundError("Snapshot '{}' not found.".format(snapshot_name))
        raise map_azure_error_to_cli_error(ex)
    return snapshot


def get_user_assigned_identity_by_resource_id(cli_ctx, resource_id):
    _re_user_assigned_identity_resource_id = re.compile(
        r"/subscriptions/(.*?)/resourcegroups/(.*?)/providers/microsoft.managedidentity/userassignedidentities/(.*)",
        flags=re.IGNORECASE,
    )
    resource_id = resource_id.lower()
    match = _re_user_assigned_identity_resource_id.search(resource_id)
    if match:
        subscription_id = match.group(1)
        resource_group_name = match.group(2)
        identity_name = match.group(3)
        return get_user_assigned_identity(cli_ctx, subscription_id, resource_group_name, identity_name)
    raise InvalidArgumentValueError("Cannot parse identity name from provided resource id '{}'.".format(resource_id))


def get_user_assigned_identity(cli_ctx, subscription_id, resource_group_name, identity_name):
    msi_client = get_msi_client(cli_ctx, subscription_id)
    try:
        identity = msi_client.user_assigned_identities.get(
            resource_group_name=resource_group_name, resource_name=identity_name
        )
    except HttpResponseError as ex:
        if "was not found" in ex.message:
            raise ResourceNotFoundError("Identity '{}' not found.".format(identity_name))
        raise ServiceError(ex.message)
    return identity


def sort_asm_revisions(revisions):
    def _convert_revision_to_semver(rev):
        sr = rev.replace("asm-", "")
        sv = sr.replace("-", ".", 1)
        # Add a custom patch version of 0
        sv += ".0"
        return semver.VersionInfo.parse(sv)

    sorted_revisions = sorted(revisions, key=_convert_revision_to_semver)
    return sorted_revisions


def _get_test_sp_client_id() -> str:
    return os.getenv("AZURE_CLI_TEST_DEV_SP_CLIENT_ID")


def _get_test_sp_object_id(sp_client_id: str) -> str:
    test_sp_client_id = _get_test_sp_client_id()
    if (
        test_sp_client_id is not None and
        sp_client_id.replace("-", "").lower() == test_sp_client_id.replace("-", "").lower()
    ):
        return os.getenv("AZURE_CLI_TEST_DEV_SP_OBJECT_ID")
    return None


def use_shared_identity() -> bool:
    return os.getenv("USE_SHARED_IDENTITY")


def _get_shared_identity(
    identity_template: str,
    identity_prefix: str,
    identity_max_id: str,
    identity_id: int = 0,
    designated_identity: str = None,
    excluded_identity: str = None,
):
    if designated_identity:
        return designated_identity

    identity = None
    if identity_template and identity_prefix:
        if identity_id:
            identity = identity_template.format(identity_prefix, str(identity_id))
        elif identity_max_id:
            tries = 0
            while tries < 10:
                try:
                    max_id = int(identity_max_id)
                    random_id = random.randint(0, max_id - 2)
                except ValueError:
                    return None
                identity = identity_template.format(identity_prefix, random_id)
                if excluded_identity and identity == excluded_identity:
                    tries += 1
                    continue
                break
    return identity


def get_shared_control_plane_identity(
    designated_identity: str = None, excluded_identity: str = None
) -> str:
    return _get_shared_identity(
        os.getenv("SHARED_CP_IDENTITY_TEMPLATE"),
        os.getenv("SHARED_CP_IDENTITY_PREFIX"),
        os.getenv("SHARED_CP_IDENTITY_MAX_ID"),
        identity_id=0,
        designated_identity=designated_identity,
        excluded_identity=excluded_identity,
    )


def get_shared_kubelet_identity(
    designated_identity: str = None,
    excluded_identity: str = None,
    shared_control_plane_identity: str = None,
    index_shift: int = 0,
) -> str:
    return _get_shared_identity(
        os.getenv("SHARED_KUBELET_IDENTITY_TEMPLATE"),
        os.getenv("SHARED_KUBELET_IDENTITY_PREFIX"),
        os.getenv("SHARED_KUBELET_IDENTITY_MAX_ID"),
        identity_id=_get_id_from_shared_control_plane_identity(
            shared_control_plane_identity
        ) + index_shift,
        designated_identity=designated_identity,
        excluded_identity=excluded_identity,
    )


def _get_id_from_shared_control_plane_identity(shared_identity) -> int:
    if (
        os.getenv("SHARED_CP_IDENTITY_TEMPLATE") and
        os.getenv("SHARED_CP_IDENTITY_PREFIX") and
        shared_identity
    ):
        return int(
            shared_identity.replace(
                os.getenv("SHARED_CP_IDENTITY_TEMPLATE").format(
                    os.getenv("SHARED_CP_IDENTITY_PREFIX"), ""
                ),
                "",
            )
        )
    return 0


def process_dns_overrides(overrides_dict, target_dict, build_override_func):
    """Helper function to safely process DNS overrides with null checks.
    Processes DNS override dictionaries from LocalDNS configuration,
    filtering out null values and applying the build function to valid entries.
    :param overrides_dict: Dictionary containing DNS overrides (can be None)
    :param target_dict: Target dictionary to populate with processed overrides
    :param build_override_func: Function to build override objects from dict values
    """
    if not isinstance(overrides_dict, dict):
        raise InvalidArgumentValueError(
            f"Expected a dictionary for DNS overrides, but got {type(overrides_dict).__name__}: {overrides_dict}"
        )
    if overrides_dict is not None:
        for key, value in overrides_dict.items():
            if value is not None:
                target_dict[key] = build_override_func(value)



# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class AllocationStrategy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the allocation strategy for the virtual machine scale set based on which the VMs will
    be allocated.
    """

    LOWEST_PRICE = "LowestPrice"
    CAPACITY_OPTIMIZED = "CapacityOptimized"
    PRIORITIZED = "Prioritized"


class AlternativeType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Describes the type of the alternative option."""

    NONE = "None"
    OFFER = "Offer"
    PLAN = "Plan"


class ArchitectureTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the Architecture Type."""

    X64 = "x64"
    ARM64 = "Arm64"


class AvailabilitySetSkuTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the sku of an Availability Set. Use 'Aligned' for virtual machines with managed disks
    and 'Classic' for virtual machines with unmanaged disks. Default value is 'Classic'.
    """

    CLASSIC = "Classic"
    ALIGNED = "Aligned"


class CachingTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the caching requirements. Possible values are: **None,** **ReadOnly,** **ReadWrite.**
    The default values are: **None for Standard storage. ReadOnly for Premium storage**.
    """

    NONE = "None"
    READ_ONLY = "ReadOnly"
    READ_WRITE = "ReadWrite"


class CapacityReservationGroupInstanceViewTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """CapacityReservationGroupInstanceViewTypes."""

    INSTANCE_VIEW = "instanceView"


class CapacityReservationInstanceViewTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """CapacityReservationInstanceViewTypes."""

    INSTANCE_VIEW = "instanceView"


class ConsistencyModeTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """ConsistencyMode of the RestorePoint. Can be specified in the input while creating a restore
    point. For now, only CrashConsistent is accepted as a valid input. Please refer to
    https://aka.ms/RestorePoints for more details.
    """

    CRASH_CONSISTENT = "CrashConsistent"
    FILE_SYSTEM_CONSISTENT = "FileSystemConsistent"
    APPLICATION_CONSISTENT = "ApplicationConsistent"


class DedicatedHostLicenseTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the software license type that will be applied to the VMs deployed on the dedicated
    host. Possible values are: **None,** **Windows_Server_Hybrid,** **Windows_Server_Perpetual.**
    The default value is: **None.**.
    """

    NONE = "None"
    WINDOWS_SERVER_HYBRID = "Windows_Server_Hybrid"
    WINDOWS_SERVER_PERPETUAL = "Windows_Server_Perpetual"


class DeleteOptions(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specify what happens to the network interface when the VM is deleted."""

    DELETE = "Delete"
    DETACH = "Detach"


class DiffDiskOptions(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the ephemeral disk option for operating system disk."""

    LOCAL = "Local"


class DiffDiskPlacement(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the ephemeral disk placement for operating system disk. This property can be used by
    user in the request to choose the location i.e, cache disk, resource disk or nvme disk space
    for Ephemeral OS disk provisioning. For more information on Ephemeral OS disk size
    requirements, please refer Ephemeral OS disk size requirements for Windows VM at
    https://docs.microsoft.com/azure/virtual-machines/windows/ephemeral-os-disks#size-requirements
    and Linux VM at
    https://docs.microsoft.com/azure/virtual-machines/linux/ephemeral-os-disks#size-requirements.
    Minimum api-version for NvmeDisk: 2024-03-01.
    """

    CACHE_DISK = "CacheDisk"
    RESOURCE_DISK = "ResourceDisk"
    NVME_DISK = "NvmeDisk"


class DiskControllerTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the disk controller type configured for the VM and VirtualMachineScaleSet. This
    property is only supported for virtual machines whose operating system disk and VM sku supports
    Generation 2 (https://docs.microsoft.com/en-us/azure/virtual-machines/generation-2), please
    check the HyperVGenerations capability returned as part of VM sku capabilities in the response
    of Microsoft.Compute SKUs api for the region contains V2
    (https://docs.microsoft.com/rest/api/compute/resourceskus/list). For more information about
    Disk Controller Types supported please refer to https://aka.ms/azure-diskcontrollertypes.
    """

    SCSI = "SCSI"
    NV_ME = "NVMe"


class DiskCreateOptionTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies how the virtual machine disk should be created. Possible values are **Attach:** This
    value is used when you are using a specialized disk to create the virtual machine.
    **FromImage:** This value is used when you are using an image to create the virtual machine. If
    you are using a platform image, you should also use the imageReference element described above.
    If you are using a marketplace image, you should also use the plan element previously
    described. **Empty:** This value is used when creating an empty data disk. **Copy:** This value
    is used to create a data disk from a snapshot or another disk. **Restore:** This value is used
    to create a data disk from a disk restore point.
    """

    FROM_IMAGE = "FromImage"
    EMPTY = "Empty"
    ATTACH = "Attach"
    COPY = "Copy"
    RESTORE = "Restore"


class DiskDeleteOptionTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the behavior of the managed disk when the VM gets deleted, for example whether the
    managed disk is deleted or detached. Supported values are: **Delete.** If this value is used,
    the managed disk is deleted when VM gets deleted. **Detach.** If this value is used, the
    managed disk is retained after VM gets deleted. Minimum api-version: 2021-03-01.
    """

    DELETE = "Delete"
    DETACH = "Detach"


class DiskDetachOptionTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the detach behavior to be used while detaching a disk or which is already in the
    process of detachment from the virtual machine. Supported values are: **ForceDetach.**
    detachOption: **ForceDetach** is applicable only for managed data disks. If a previous
    detachment attempt of the data disk did not complete due to an unexpected failure from the
    virtual machine and the disk is still not released then use force-detach as a last resort
    option to detach the disk forcibly from the VM. All writes might not have been flushed when
    using this detach behavior. To force-detach a data disk update toBeDetached to 'true' along
    with setting detachOption: 'ForceDetach'.
    """

    FORCE_DETACH = "ForceDetach"


class DomainNameLabelScopeTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The Domain name label scope.The concatenation of the hashed domain name label that generated
    according to the policy from domain name label scope and vm index will be the domain name
    labels of the PublicIPAddress resources that will be created.
    """

    TENANT_REUSE = "TenantReuse"
    SUBSCRIPTION_REUSE = "SubscriptionReuse"
    RESOURCE_GROUP_REUSE = "ResourceGroupReuse"
    NO_REUSE = "NoReuse"


class ExecutionState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Script execution status."""

    UNKNOWN = "Unknown"
    PENDING = "Pending"
    RUNNING = "Running"
    FAILED = "Failed"
    SUCCEEDED = "Succeeded"
    TIMED_OUT = "TimedOut"
    CANCELED = "Canceled"


class Expand(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Expand."""

    PROPERTIES = "Properties"


class ExpandTypeForListVMs(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """ExpandTypeForListVMs."""

    INSTANCE_VIEW = "instanceView"


class ExpandTypesForGetCapacityReservationGroups(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """ExpandTypesForGetCapacityReservationGroups."""

    VIRTUAL_MACHINE_SCALE_SET_VMS_REF = "virtualMachineScaleSetVMs/$ref"
    VIRTUAL_MACHINES_REF = "virtualMachines/$ref"


class ExpandTypesForGetVMScaleSets(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """ExpandTypesForGetVMScaleSets."""

    USER_DATA = "userData"


class ExpandTypesForListVMs(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """ExpandTypesForListVMs."""

    INSTANCE_VIEW = "instanceView"


class ExtendedLocationTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of extendedLocation."""

    EDGE_ZONE = "EdgeZone"


class HyperVGeneration(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The hypervisor generation of the Virtual Machine [V1, V2]."""

    V1 = "V1"
    V2 = "V2"


class HyperVGenerationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the HyperVGeneration Type associated with a resource."""

    V1 = "V1"
    V2 = "V2"


class HyperVGenerationTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the HyperVGeneration Type."""

    V1 = "V1"
    V2 = "V2"


class ImageState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Describes the state of the image."""

    ACTIVE = "Active"
    SCHEDULED_FOR_DEPRECATION = "ScheduledForDeprecation"
    DEPRECATED = "Deprecated"


class InstanceViewTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """InstanceViewTypes."""

    INSTANCE_VIEW = "instanceView"
    USER_DATA = "userData"
    RESILIENCY_VIEW = "resiliencyView"


class IntervalInMins(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Interval value in minutes used to create LogAnalytics call rate logs."""

    THREE_MINS = "ThreeMins"
    FIVE_MINS = "FiveMins"
    THIRTY_MINS = "ThirtyMins"
    SIXTY_MINS = "SixtyMins"


class IPVersion(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Available from Api-Version 2017-03-30 onwards, it represents whether the specific
    ipconfiguration is IPv4 or IPv6. Default is taken as IPv4.  Possible values are: 'IPv4' and
    'IPv6'.
    """

    I_PV4 = "IPv4"
    I_PV6 = "IPv6"


class IPVersions(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Available from Api-Version 2019-07-01 onwards, it represents whether the specific
    ipconfiguration is IPv4 or IPv6. Default is taken as IPv4. Possible values are: 'IPv4' and
    'IPv6'.
    """

    I_PV4 = "IPv4"
    I_PV6 = "IPv6"


class LinuxPatchAssessmentMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the mode of VM Guest Patch Assessment for the IaaS virtual machine.\\ :code:`<br
    />`\\ :code:`<br />` Possible values are:\\ :code:`<br />`\\ :code:`<br />` **ImageDefault** -
    You control the timing of patch assessments on a virtual machine. :code:`<br />`\\ :code:`<br
    />` **AutomaticByPlatform** - The platform will trigger periodic patch assessments. The
    property provisionVMAgent must be true.
    """

    IMAGE_DEFAULT = "ImageDefault"
    AUTOMATIC_BY_PLATFORM = "AutomaticByPlatform"


class LinuxVMGuestPatchAutomaticByPlatformRebootSetting(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the reboot setting for all AutomaticByPlatform patch installation operations."""

    UNKNOWN = "Unknown"
    IF_REQUIRED = "IfRequired"
    NEVER = "Never"
    ALWAYS = "Always"


class LinuxVMGuestPatchMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the mode of VM Guest Patching to IaaS virtual machine or virtual machines associated
    to virtual machine scale set with OrchestrationMode as Flexible.\\ :code:`<br />`\\ :code:`<br
    />` Possible values are:\\ :code:`<br />`\\ :code:`<br />` **ImageDefault** - The virtual
    machine's default patching configuration is used. :code:`<br />`\\ :code:`<br />`
    **AutomaticByPlatform** - The virtual machine will be automatically updated by the platform.
    The property provisionVMAgent must be true.
    """

    IMAGE_DEFAULT = "ImageDefault"
    AUTOMATIC_BY_PLATFORM = "AutomaticByPlatform"


class MaintenanceOperationResultCodeTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The Last Maintenance Operation Result Code."""

    NONE = "None"
    RETRY_LATER = "RetryLater"
    MAINTENANCE_ABORTED = "MaintenanceAborted"
    MAINTENANCE_COMPLETED = "MaintenanceCompleted"


class Mode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the mode that ProxyAgent will execute on. Warning: this property has been deprecated,
    please specify 'mode' under particular hostendpoint setting.
    """

    AUDIT = "Audit"
    ENFORCE = "Enforce"


class Modes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the execution mode. In Audit mode, the system acts as if it is enforcing the access
    control policy, including emitting access denial entries in the logs but it does not actually
    deny any requests to host endpoints. In Enforce mode, the system will enforce the access
    control and it is the recommended mode of operation.
    """

    AUDIT = "Audit"
    ENFORCE = "Enforce"
    DISABLED = "Disabled"


class NetworkApiVersion(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """specifies the Microsoft.Network API version used when creating networking resources in the
    Network Interface Configurations.
    """

    TWO_THOUSAND_TWENTY11_01 = "2020-11-01"
    TWO_THOUSAND_TWENTY_TWO11_01 = "2022-11-01"


class NetworkInterfaceAuxiliaryMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies whether the Auxiliary mode is enabled for the Network Interface resource."""

    NONE = "None"
    ACCELERATED_CONNECTIONS = "AcceleratedConnections"
    FLOATING = "Floating"


class NetworkInterfaceAuxiliarySku(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies whether the Auxiliary sku is enabled for the Network Interface resource."""

    NONE = "None"
    A1 = "A1"
    A2 = "A2"
    A4 = "A4"
    A8 = "A8"


class OperatingSystemStateTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The OS State. For managed images, use Generalized."""

    GENERALIZED = "Generalized"
    """Generalized image. Needs to be provisioned during deployment time."""
    SPECIALIZED = "Specialized"
    """Specialized image. Contains already provisioned OS Disk."""


class OperatingSystemType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Gets the Operating System type."""

    WINDOWS = "Windows"
    LINUX = "Linux"


class OperatingSystemTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """This property allows you to specify the type of the OS that is included in the disk if creating
    a VM from user-image or a specialized VHD. Possible values are: **Windows,** **Linux.**.
    """

    WINDOWS = "Windows"
    LINUX = "Linux"


class OrchestrationMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the orchestration mode for the virtual machine scale set."""

    UNIFORM = "Uniform"
    FLEXIBLE = "Flexible"


class OrchestrationServiceNames(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The name of the service."""

    AUTOMATIC_REPAIRS = "AutomaticRepairs"
    DUMMY_ORCHESTRATION_SERVICE_NAME = "DummyOrchestrationServiceName"


class OrchestrationServiceState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The current state of the service."""

    NOT_RUNNING = "NotRunning"
    RUNNING = "Running"
    SUSPENDED = "Suspended"


class OrchestrationServiceStateAction(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The action to be performed."""

    RESUME = "Resume"
    SUSPEND = "Suspend"


class PatchAssessmentState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Describes the availability of a given patch."""

    UNKNOWN = "Unknown"
    AVAILABLE = "Available"


class PatchInstallationState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The state of the patch after the installation operation completed."""

    UNKNOWN = "Unknown"
    INSTALLED = "Installed"
    FAILED = "Failed"
    EXCLUDED = "Excluded"
    NOT_SELECTED = "NotSelected"
    PENDING = "Pending"


class PatchOperationStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The overall success or failure status of the operation. It remains "InProgress" until the
    operation completes. At that point it will become "Unknown", "Failed", "Succeeded", or
    "CompletedWithWarnings.".
    """

    UNKNOWN = "Unknown"
    IN_PROGRESS = "InProgress"
    FAILED = "Failed"
    SUCCEEDED = "Succeeded"
    COMPLETED_WITH_WARNINGS = "CompletedWithWarnings"


class ProtocolTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the protocol of WinRM listener. Possible values are: **http,** **https.**."""

    HTTP = "Http"
    HTTPS = "Https"


class ProximityPlacementGroupType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the type of the proximity placement group. Possible values are: **Standard** :
    Co-locate resources within an Azure region or Availability Zone. **Ultra** : For future use.
    """

    STANDARD = "Standard"
    ULTRA = "Ultra"


class PublicIPAddressSkuName(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specify public IP sku name."""

    BASIC = "Basic"
    STANDARD = "Standard"


class PublicIPAddressSkuTier(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specify public IP sku tier."""

    REGIONAL = "Regional"
    GLOBAL = "Global"
    GLOBAL_ENUM = "Global"


class PublicIPAllocationMethod(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specify the public IP allocation type."""

    DYNAMIC = "Dynamic"
    STATIC = "Static"


class RebalanceBehavior(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of rebalance behavior that will be used for recreating virtual machines in the scale set
    across availability zones. Default and only supported value for now is CreateBeforeDelete.
    """

    CREATE_BEFORE_DELETE = "CreateBeforeDelete"


class RebalanceStrategy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of rebalance strategy that will be used for rebalancing virtual machines in the scale set
    across availability zones. Default and only supported value for now is Recreate.
    """

    RECREATE = "Recreate"


class RepairAction(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of repair action (replace, restart, reimage) that will be used for repairing unhealthy
    virtual machines in the scale set. Default value is replace.
    """

    REPLACE = "Replace"
    RESTART = "Restart"
    REIMAGE = "Reimage"


class ResilientVMDeletionStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the resilient VM deletion status for the virtual machine."""

    ENABLED = "Enabled"
    DISABLED = "Disabled"
    IN_PROGRESS = "InProgress"
    FAILED = "Failed"


class ResourceIdentityType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of identity used for the virtual machine scale set. The type 'SystemAssigned,
    UserAssigned' includes both an implicitly created identity and a set of user assigned
    identities. The type 'None' will remove any identities from the virtual machine scale set.
    """

    SYSTEM_ASSIGNED = "SystemAssigned"
    USER_ASSIGNED = "UserAssigned"
    SYSTEM_ASSIGNED_USER_ASSIGNED = "SystemAssigned, UserAssigned"
    NONE = "None"


class ResourceIdOptionsForGetCapacityReservationGroups(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """ResourceIdOptionsForGetCapacityReservationGroups."""

    CREATED_IN_SUBSCRIPTION = "CreatedInSubscription"
    SHARED_WITH_SUBSCRIPTION = "SharedWithSubscription"
    ALL = "All"


class RestorePointCollectionExpandOptions(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """RestorePointCollectionExpandOptions."""

    RESTORE_POINTS = "restorePoints"


class RestorePointEncryptionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of key used to encrypt the data of the disk restore point."""

    ENCRYPTION_AT_REST_WITH_PLATFORM_KEY = "EncryptionAtRestWithPlatformKey"
    """Disk Restore Point is encrypted at rest with Platform managed key."""
    ENCRYPTION_AT_REST_WITH_CUSTOMER_KEY = "EncryptionAtRestWithCustomerKey"
    """Disk Restore Point is encrypted at rest with Customer managed key that can be changed and
    revoked by a customer."""
    ENCRYPTION_AT_REST_WITH_PLATFORM_AND_CUSTOMER_KEYS = "EncryptionAtRestWithPlatformAndCustomerKeys"
    """Disk Restore Point is encrypted at rest with 2 layers of encryption. One of the keys is
    Customer managed and the other key is Platform managed."""


class RestorePointExpandOptions(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """RestorePointExpandOptions."""

    INSTANCE_VIEW = "instanceView"


class RollingUpgradeActionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The last action performed on the rolling upgrade."""

    START = "Start"
    CANCEL = "Cancel"


class RollingUpgradeStatusCode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Code indicating the current status of the upgrade."""

    ROLLING_FORWARD = "RollingForward"
    CANCELLED = "Cancelled"
    COMPLETED = "Completed"
    FAULTED = "Faulted"


class SecurityEncryptionTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the EncryptionType of the managed disk. It is set to DiskWithVMGuestState for
    encryption of the managed disk along with VMGuestState blob, VMGuestStateOnly for encryption of
    just the VMGuestState blob, and NonPersistedTPM for not persisting firmware state in the
    VMGuestState blob.. **Note:** It can be set for only Confidential VMs.
    """

    VM_GUEST_STATE_ONLY = "VMGuestStateOnly"
    DISK_WITH_VM_GUEST_STATE = "DiskWithVMGuestState"
    NON_PERSISTED_TPM = "NonPersistedTPM"


class SecurityTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the SecurityType of the virtual machine. It has to be set to any specified value to
    enable UefiSettings. The default behavior is: UefiSettings will not be enabled unless this
    property is set.
    """

    TRUSTED_LAUNCH = "TrustedLaunch"
    CONFIDENTIAL_VM = "ConfidentialVM"


class SettingNames(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the name of the setting to which the content applies. Possible values are:
    FirstLogonCommands and AutoLogon.
    """

    AUTO_LOGON = "AutoLogon"
    FIRST_LOGON_COMMANDS = "FirstLogonCommands"


class SshEncryptionTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The encryption type of the SSH keys to be generated. See SshEncryptionTypes for possible set of
    values. If not provided, will default to RSA.
    """

    RSA = "RSA"
    ED25519 = "Ed25519"


class StatusLevelTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The level code."""

    INFO = "Info"
    WARNING = "Warning"
    ERROR = "Error"


class StorageAccountTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the storage account type for the managed disk. Managed OS disk storage account type
    can only be set when you create the scale set. NOTE: UltraSSD_LRS can only be used with data
    disks. It cannot be used with OS Disk. Standard_LRS uses Standard HDD. StandardSSD_LRS uses
    Standard SSD. Premium_LRS uses Premium SSD. UltraSSD_LRS uses Ultra disk. Premium_ZRS uses
    Premium SSD zone redundant storage. StandardSSD_ZRS uses Standard SSD zone redundant storage.
    For more information regarding disks supported for Windows Virtual Machines, refer to
    https://docs.microsoft.com/azure/virtual-machines/windows/disks-types and, for Linux Virtual
    Machines, refer to https://docs.microsoft.com/azure/virtual-machines/linux/disks-types.
    """

    STANDARD_LRS = "Standard_LRS"
    PREMIUM_LRS = "Premium_LRS"
    STANDARD_SSD_LRS = "StandardSSD_LRS"
    ULTRA_SSD_LRS = "UltraSSD_LRS"
    PREMIUM_ZRS = "Premium_ZRS"
    STANDARD_SSD_ZRS = "StandardSSD_ZRS"
    PREMIUM_V2_LRS = "PremiumV2_LRS"


class UpgradeMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the mode of an upgrade to virtual machines in the scale set.\\ :code:`<br />`\\
    :code:`<br />` Possible values are:\\ :code:`<br />`\\ :code:`<br />` **Manual** - You  control
    the application of updates to virtual machines in the scale set. You do this by using the
    manualUpgrade action.\\ :code:`<br />`\\ :code:`<br />` **Automatic** - All virtual machines in
    the scale set are  automatically updated at the same time.
    """

    AUTOMATIC = "Automatic"
    MANUAL = "Manual"
    ROLLING = "Rolling"


class UpgradeOperationInvoker(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Invoker of the Upgrade Operation."""

    UNKNOWN = "Unknown"
    USER = "User"
    PLATFORM = "Platform"


class UpgradeState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Code indicating the current status of the upgrade."""

    ROLLING_FORWARD = "RollingForward"
    CANCELLED = "Cancelled"
    COMPLETED = "Completed"
    FAULTED = "Faulted"


class VirtualMachineEvictionPolicyTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the eviction policy for the Azure Spot VM/VMSS."""

    DEALLOCATE = "Deallocate"
    DELETE = "Delete"


class VirtualMachinePriorityTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the priority for a standalone virtual machine or the virtual machines in the scale
    set. 'Low' enum will be deprecated in the future, please use 'Spot' as the enum to deploy Azure
    Spot VM/VMSS.
    """

    REGULAR = "Regular"
    LOW = "Low"
    SPOT = "Spot"


class VirtualMachineScaleSetScaleInRules(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """VirtualMachineScaleSetScaleInRules."""

    DEFAULT = "Default"
    OLDEST_VM = "OldestVM"
    NEWEST_VM = "NewestVM"


class VirtualMachineScaleSetSkuScaleType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The scale type applicable to the sku."""

    AUTOMATIC = "Automatic"
    NONE = "None"


class VirtualMachineSizeTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the size of the virtual machine. The enum data type is currently deprecated and will
    be removed by December 23rd 2023. The recommended way to get the list of available sizes is
    using these APIs: `List all available virtual machine sizes in an availability set
    <https://docs.microsoft.com/rest/api/compute/availabilitysets/listavailablesizes>`_\\ , `List
    all available virtual machine sizes in a region
    <https://docs.microsoft.com/rest/api/compute/resourceskus/list>`_\\ , `List all available
    virtual machine sizes for resizing
    <https://docs.microsoft.com/rest/api/compute/virtualmachines/listavailablesizes>`_. For more
    information about virtual machine sizes, see `Sizes for virtual machines
    <https://docs.microsoft.com/azure/virtual-machines/sizes>`_. The available VM sizes depend on
    region and availability set.
    """

    BASIC_A0 = "Basic_A0"
    BASIC_A1 = "Basic_A1"
    BASIC_A2 = "Basic_A2"
    BASIC_A3 = "Basic_A3"
    BASIC_A4 = "Basic_A4"
    STANDARD_A0 = "Standard_A0"
    STANDARD_A1 = "Standard_A1"
    STANDARD_A2 = "Standard_A2"
    STANDARD_A3 = "Standard_A3"
    STANDARD_A4 = "Standard_A4"
    STANDARD_A5 = "Standard_A5"
    STANDARD_A6 = "Standard_A6"
    STANDARD_A7 = "Standard_A7"
    STANDARD_A8 = "Standard_A8"
    STANDARD_A9 = "Standard_A9"
    STANDARD_A10 = "Standard_A10"
    STANDARD_A11 = "Standard_A11"
    STANDARD_A1_V2 = "Standard_A1_v2"
    STANDARD_A2_V2 = "Standard_A2_v2"
    STANDARD_A4_V2 = "Standard_A4_v2"
    STANDARD_A8_V2 = "Standard_A8_v2"
    STANDARD_A2_M_V2 = "Standard_A2m_v2"
    STANDARD_A4_M_V2 = "Standard_A4m_v2"
    STANDARD_A8_M_V2 = "Standard_A8m_v2"
    STANDARD_B1_S = "Standard_B1s"
    STANDARD_B1_MS = "Standard_B1ms"
    STANDARD_B2_S = "Standard_B2s"
    STANDARD_B2_MS = "Standard_B2ms"
    STANDARD_B4_MS = "Standard_B4ms"
    STANDARD_B8_MS = "Standard_B8ms"
    STANDARD_D1 = "Standard_D1"
    STANDARD_D2 = "Standard_D2"
    STANDARD_D3 = "Standard_D3"
    STANDARD_D4 = "Standard_D4"
    STANDARD_D11 = "Standard_D11"
    STANDARD_D12 = "Standard_D12"
    STANDARD_D13 = "Standard_D13"
    STANDARD_D14 = "Standard_D14"
    STANDARD_D1_V2 = "Standard_D1_v2"
    STANDARD_D2_V2 = "Standard_D2_v2"
    STANDARD_D3_V2 = "Standard_D3_v2"
    STANDARD_D4_V2 = "Standard_D4_v2"
    STANDARD_D5_V2 = "Standard_D5_v2"
    STANDARD_D2_V3 = "Standard_D2_v3"
    STANDARD_D4_V3 = "Standard_D4_v3"
    STANDARD_D8_V3 = "Standard_D8_v3"
    STANDARD_D16_V3 = "Standard_D16_v3"
    STANDARD_D32_V3 = "Standard_D32_v3"
    STANDARD_D64_V3 = "Standard_D64_v3"
    STANDARD_D2_S_V3 = "Standard_D2s_v3"
    STANDARD_D4_S_V3 = "Standard_D4s_v3"
    STANDARD_D8_S_V3 = "Standard_D8s_v3"
    STANDARD_D16_S_V3 = "Standard_D16s_v3"
    STANDARD_D32_S_V3 = "Standard_D32s_v3"
    STANDARD_D64_S_V3 = "Standard_D64s_v3"
    STANDARD_D11_V2 = "Standard_D11_v2"
    STANDARD_D12_V2 = "Standard_D12_v2"
    STANDARD_D13_V2 = "Standard_D13_v2"
    STANDARD_D14_V2 = "Standard_D14_v2"
    STANDARD_D15_V2 = "Standard_D15_v2"
    STANDARD_DS1 = "Standard_DS1"
    STANDARD_DS2 = "Standard_DS2"
    STANDARD_DS3 = "Standard_DS3"
    STANDARD_DS4 = "Standard_DS4"
    STANDARD_DS11 = "Standard_DS11"
    STANDARD_DS12 = "Standard_DS12"
    STANDARD_DS13 = "Standard_DS13"
    STANDARD_DS14 = "Standard_DS14"
    STANDARD_DS1_V2 = "Standard_DS1_v2"
    STANDARD_DS2_V2 = "Standard_DS2_v2"
    STANDARD_DS3_V2 = "Standard_DS3_v2"
    STANDARD_DS4_V2 = "Standard_DS4_v2"
    STANDARD_DS5_V2 = "Standard_DS5_v2"
    STANDARD_DS11_V2 = "Standard_DS11_v2"
    STANDARD_DS12_V2 = "Standard_DS12_v2"
    STANDARD_DS13_V2 = "Standard_DS13_v2"
    STANDARD_DS14_V2 = "Standard_DS14_v2"
    STANDARD_DS15_V2 = "Standard_DS15_v2"
    STANDARD_DS13_4_V2 = "Standard_DS13-4_v2"
    STANDARD_DS13_2_V2 = "Standard_DS13-2_v2"
    STANDARD_DS14_8_V2 = "Standard_DS14-8_v2"
    STANDARD_DS14_4_V2 = "Standard_DS14-4_v2"
    STANDARD_E2_V3 = "Standard_E2_v3"
    STANDARD_E4_V3 = "Standard_E4_v3"
    STANDARD_E8_V3 = "Standard_E8_v3"
    STANDARD_E16_V3 = "Standard_E16_v3"
    STANDARD_E32_V3 = "Standard_E32_v3"
    STANDARD_E64_V3 = "Standard_E64_v3"
    STANDARD_E2_S_V3 = "Standard_E2s_v3"
    STANDARD_E4_S_V3 = "Standard_E4s_v3"
    STANDARD_E8_S_V3 = "Standard_E8s_v3"
    STANDARD_E16_S_V3 = "Standard_E16s_v3"
    STANDARD_E32_S_V3 = "Standard_E32s_v3"
    STANDARD_E64_S_V3 = "Standard_E64s_v3"
    STANDARD_E32_16_V3 = "Standard_E32-16_v3"
    STANDARD_E32_8_S_V3 = "Standard_E32-8s_v3"
    STANDARD_E64_32_S_V3 = "Standard_E64-32s_v3"
    STANDARD_E64_16_S_V3 = "Standard_E64-16s_v3"
    STANDARD_F1 = "Standard_F1"
    STANDARD_F2 = "Standard_F2"
    STANDARD_F4 = "Standard_F4"
    STANDARD_F8 = "Standard_F8"
    STANDARD_F16 = "Standard_F16"
    STANDARD_F1_S = "Standard_F1s"
    STANDARD_F2_S = "Standard_F2s"
    STANDARD_F4_S = "Standard_F4s"
    STANDARD_F8_S = "Standard_F8s"
    STANDARD_F16_S = "Standard_F16s"
    STANDARD_F2_S_V2 = "Standard_F2s_v2"
    STANDARD_F4_S_V2 = "Standard_F4s_v2"
    STANDARD_F8_S_V2 = "Standard_F8s_v2"
    STANDARD_F16_S_V2 = "Standard_F16s_v2"
    STANDARD_F32_S_V2 = "Standard_F32s_v2"
    STANDARD_F64_S_V2 = "Standard_F64s_v2"
    STANDARD_F72_S_V2 = "Standard_F72s_v2"
    STANDARD_G1 = "Standard_G1"
    STANDARD_G2 = "Standard_G2"
    STANDARD_G3 = "Standard_G3"
    STANDARD_G4 = "Standard_G4"
    STANDARD_G5 = "Standard_G5"
    STANDARD_GS1 = "Standard_GS1"
    STANDARD_GS2 = "Standard_GS2"
    STANDARD_GS3 = "Standard_GS3"
    STANDARD_GS4 = "Standard_GS4"
    STANDARD_GS5 = "Standard_GS5"
    STANDARD_GS4_8 = "Standard_GS4-8"
    STANDARD_GS4_4 = "Standard_GS4-4"
    STANDARD_GS5_16 = "Standard_GS5-16"
    STANDARD_GS5_8 = "Standard_GS5-8"
    STANDARD_H8 = "Standard_H8"
    STANDARD_H16 = "Standard_H16"
    STANDARD_H8_M = "Standard_H8m"
    STANDARD_H16_M = "Standard_H16m"
    STANDARD_H16_R = "Standard_H16r"
    STANDARD_H16_MR = "Standard_H16mr"
    STANDARD_L4_S = "Standard_L4s"
    STANDARD_L8_S = "Standard_L8s"
    STANDARD_L16_S = "Standard_L16s"
    STANDARD_L32_S = "Standard_L32s"
    STANDARD_M64_S = "Standard_M64s"
    STANDARD_M64_MS = "Standard_M64ms"
    STANDARD_M128_S = "Standard_M128s"
    STANDARD_M128_MS = "Standard_M128ms"
    STANDARD_M64_32_MS = "Standard_M64-32ms"
    STANDARD_M64_16_MS = "Standard_M64-16ms"
    STANDARD_M128_64_MS = "Standard_M128-64ms"
    STANDARD_M128_32_MS = "Standard_M128-32ms"
    STANDARD_NC6 = "Standard_NC6"
    STANDARD_NC12 = "Standard_NC12"
    STANDARD_NC24 = "Standard_NC24"
    STANDARD_NC24_R = "Standard_NC24r"
    STANDARD_NC6_S_V2 = "Standard_NC6s_v2"
    STANDARD_NC12_S_V2 = "Standard_NC12s_v2"
    STANDARD_NC24_S_V2 = "Standard_NC24s_v2"
    STANDARD_NC24_RS_V2 = "Standard_NC24rs_v2"
    STANDARD_NC6_S_V3 = "Standard_NC6s_v3"
    STANDARD_NC12_S_V3 = "Standard_NC12s_v3"
    STANDARD_NC24_S_V3 = "Standard_NC24s_v3"
    STANDARD_NC24_RS_V3 = "Standard_NC24rs_v3"
    STANDARD_ND6_S = "Standard_ND6s"
    STANDARD_ND12_S = "Standard_ND12s"
    STANDARD_ND24_S = "Standard_ND24s"
    STANDARD_ND24_RS = "Standard_ND24rs"
    STANDARD_NV6 = "Standard_NV6"
    STANDARD_NV12 = "Standard_NV12"
    STANDARD_NV24 = "Standard_NV24"


class VmDiskTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """VM disk types which are disallowed."""

    NONE = "None"
    UNMANAGED = "Unmanaged"


class VMGuestPatchClassificationLinux(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """VMGuestPatchClassificationLinux."""

    CRITICAL = "Critical"
    SECURITY = "Security"
    OTHER = "Other"


class VMGuestPatchClassificationWindows(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """VMGuestPatchClassificationWindows."""

    CRITICAL = "Critical"
    SECURITY = "Security"
    UPDATE_ROLL_UP = "UpdateRollUp"
    FEATURE_PACK = "FeaturePack"
    SERVICE_PACK = "ServicePack"
    DEFINITION = "Definition"
    TOOLS = "Tools"
    UPDATES = "Updates"


class VMGuestPatchRebootBehavior(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Describes the reboot requirements of the patch."""

    UNKNOWN = "Unknown"
    NEVER_REBOOTS = "NeverReboots"
    ALWAYS_REQUIRES_REBOOT = "AlwaysRequiresReboot"
    CAN_REQUEST_REBOOT = "CanRequestReboot"


class VMGuestPatchRebootSetting(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Defines when it is acceptable to reboot a VM during a software update operation."""

    IF_REQUIRED = "IfRequired"
    NEVER = "Never"
    ALWAYS = "Always"


class VMGuestPatchRebootStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The reboot state of the VM following completion of the operation."""

    UNKNOWN = "Unknown"
    NOT_NEEDED = "NotNeeded"
    REQUIRED = "Required"
    STARTED = "Started"
    FAILED = "Failed"
    COMPLETED = "Completed"


class WindowsPatchAssessmentMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the mode of VM Guest patch assessment for the IaaS virtual machine.\\ :code:`<br
    />`\\ :code:`<br />` Possible values are:\\ :code:`<br />`\\ :code:`<br />` **ImageDefault** -
    You control the timing of patch assessments on a virtual machine.\\ :code:`<br />`\\ :code:`<br
    />` **AutomaticByPlatform** - The platform will trigger periodic patch assessments. The
    property provisionVMAgent must be true.
    """

    IMAGE_DEFAULT = "ImageDefault"
    AUTOMATIC_BY_PLATFORM = "AutomaticByPlatform"


class WindowsVMGuestPatchAutomaticByPlatformRebootSetting(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the reboot setting for all AutomaticByPlatform patch installation operations."""

    UNKNOWN = "Unknown"
    IF_REQUIRED = "IfRequired"
    NEVER = "Never"
    ALWAYS = "Always"


class WindowsVMGuestPatchMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the mode of VM Guest Patching to IaaS virtual machine or virtual machines associated
    to virtual machine scale set with OrchestrationMode as Flexible.\\ :code:`<br />`\\ :code:`<br
    />` Possible values are:\\ :code:`<br />`\\ :code:`<br />` **Manual** - You  control the
    application of patches to a virtual machine. You do this by applying patches manually inside
    the VM. In this mode, automatic updates are disabled; the property
    WindowsConfiguration.enableAutomaticUpdates must be false\\ :code:`<br />`\\ :code:`<br />`
    **AutomaticByOS** - The virtual machine will automatically be updated by the OS. The property
    WindowsConfiguration.enableAutomaticUpdates must be true. :code:`<br />`\\ :code:`<br />`
    **AutomaticByPlatform** - the virtual machine will automatically updated by the platform. The
    properties provisionVMAgent and WindowsConfiguration.enableAutomaticUpdates must be true.
    """

    MANUAL = "Manual"
    AUTOMATIC_BY_OS = "AutomaticByOS"
    AUTOMATIC_BY_PLATFORM = "AutomaticByPlatform"


class ZonalPlatformFaultDomainAlignMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the align mode between Virtual Machine Scale Set compute and storage Fault Domain
    count.
    """

    ALIGNED = "Aligned"
    UNALIGNED = "Unaligned"


class ZonePlacementPolicyType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the policy for virtual machine's placement in availability zone. Possible values are:
    **Any** - An availability zone will be automatically picked by system as part of virtual
    machine creation.
    """

    ANY = "Any"



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=protected-access

from ._arg import AAZListArg, AAZDictArg
from ._base import has_value
from ._field_value import AAZList, AAZDict
from ._base import AAZBaseValue


def assign_aaz_list_arg(target: AAZList, source: AAZList, element_transformer=None):
    """ an convenience function to transform from a source AAZListArg to a target AAZListArg
    Example:

    args.target = assign_aaz_list_arg(args.target, args.source, lambda idx, e: {'idx': idx, "prop": e})
    """
    assert isinstance(target, AAZBaseValue)
    assert isinstance(source, AAZBaseValue)

    target_schema = target._schema
    source_schema = source._schema
    assert isinstance(target_schema, AAZListArg)
    assert isinstance(source_schema, AAZListArg)

    assert target_schema._nullable == source_schema._nullable, "Inconsist nullable property between target and " \
                                                               "source args"
    assert target_schema.Element._nullable == source_schema.Element._nullable, "Inconsist nullable property between " \
                                                                               "target element and source arg element"

    if not has_value(source):
        return target

    if source == None:  # noqa: E711, pylint: disable=singleton-comparison
        return None

    if not source._is_patch:
        # return the whole array
        data = []
        for idx, element in enumerate(source):
            if not has_value(element):
                continue
            if element == None:  # noqa: E711, pylint: disable=singleton-comparison
                data.append(None)
            elif element_transformer:
                data.append(element_transformer(idx, element))
            else:
                data.append(element)
        return data

    # assign by patch way
    for idx, element in enumerate(source):
        if not has_value(element):
            continue
        if element == None:  # noqa: E711, pylint: disable=singleton-comparison
            target[idx] = None
        elif element_transformer:
            target[idx] = element_transformer(idx, element)
        else:
            target[idx] = element

    return target


def assign_aaz_dict_arg(target: AAZDict, source: AAZDict, element_transformer=None):
    """ an convenience function to transform from a source AAZDictArg to a target AAZDictArg
    Example:

    args.target = assign_aaz_dict_arg(args.target, args.source, lambda key, e: {'name': key, "prop": e})
    """

    assert isinstance(target, AAZBaseValue)
    assert isinstance(source, AAZBaseValue)

    target_schema = target._schema
    source_schema = source._schema
    assert isinstance(target_schema, AAZDictArg)
    assert isinstance(source_schema, AAZDictArg)

    assert target_schema._nullable == source_schema._nullable, "Inconsist nullable property between target and " \
                                                               "source args"
    assert target_schema.Element._nullable == source_schema.Element._nullable, "Inconsist nullable property between " \
                                                                               "target element and source arg element"

    if not has_value(source):
        return target

    if source == None:  # noqa: E711, pylint: disable=singleton-comparison
        return None

    if not source._is_patch:
        # return the whole array
        data = {}
        for key, element in source.items():
            if not has_value(element):
                continue
            if element == None:  # noqa: E711, pylint: disable=singleton-comparison
                data[key] = None
            elif element_transformer:
                data[key] = element_transformer(key, element)
            else:
                data[key] = element
        return data

    # assign by patch way
    for key, element in source.items():
        if not has_value(element):
            continue
        if element == None:  # noqa: E711, pylint: disable=singleton-comparison
            target[key] = None
        elif element_transformer:
            target[key] = element_transformer(key, element)
        else:
            target[key] = element

    return target


def get_aaz_profile_module_name(profile_name):
    profile_module_name = profile_name.lower().replace('-', '_')
    if profile_module_name != "latest":
        # the rest profiles for azure-stack use start with digit number, it's not a valid python package name.
        profile_module_name = "profile_" + profile_module_name
    return profile_module_name



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=unused-argument, line-too-long
from azure.cli.core.util import user_confirmation
from knack.util import CLIError
from .._client_factory import cf_postgres_flexible_replica
from ..utils._flexible_server_location_capabilities_util import get_postgres_server_capability_info
from ..utils._flexible_server_util import resolve_poller
from ..utils.validators import pg_version_validator, validate_citus_cluster, validate_resource_group


def flexible_server_version_upgrade(cmd, client, resource_group_name, server_name, version, yes=None):
    validate_resource_group(resource_group_name)
    validate_citus_cluster(cmd, resource_group_name, server_name)

    if not yes:
        user_confirmation(
            "Upgrading major version in server {} is irreversible. The action you're about to take can't be undone. "
            "Going further will initiate major version upgrade to the selected version on this server."
            .format(server_name), yes=yes)

    instance = client.get(resource_group_name, server_name)

    if instance and instance.storage.type == "PremiumV2_LRS" and version and int(version) < 14:
        raise CLIError('Storage type PremiumV2_LRS is only supported for PostgreSQL version 14 and above.')

    current_version = int(instance.version.split('.')[0])
    if current_version >= int(version):
        raise CLIError("The version to upgrade to must be greater than the current version.")

    list_server_capability_info = get_postgres_server_capability_info(cmd, resource_group_name, server_name)
    eligible_versions = list_server_capability_info['supported_server_versions'][str(current_version)]

    pg_version_validator(version, eligible_versions)

    if version not in eligible_versions:
        # version not supported
        error_message = ""
        if len(eligible_versions) > 0:
            error_message = "Server is running version {}. It can only be upgraded to the following versions: {} ".format(str(current_version), eligible_versions)
        else:
            error_message = "Server is running version {}. It cannot be upgraded to any higher version. ".format(str(current_version))

        raise CLIError(error_message)

    replica_operations_client = cf_postgres_flexible_replica(cmd.cli_ctx, '_')
    version_mapped = version

    replicas = replica_operations_client.list_by_server(resource_group_name, server_name)

    if 'replica' in instance.replication_role.lower() or len(list(replicas)) > 0:
        raise CLIError("Major version upgrade is not yet supported for servers in a read replica setup.")

    parameters = {
        'properties': {
            'version': version_mapped
        }
    }

    return resolve_poller(
        client.begin_update(
            resource_group_name=resource_group_name,
            server_name=server_name,
            parameters=parameters),
        cmd.cli_ctx, 'Upgrading server {} to major version {}'.format(server_name, version)
    )
